/*
 * Copyright 2013 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef GNURADIO_CONFIG_H
#define GNURADIO_CONFIG_H
#ifndef TRY_SHM_VMCIRCBUF
/* #undef TRY_SHM_VMCIRCBUF */
#endif
#ifndef GR_PERFORMANCE_COUNTERS
#define GR_PERFORMANCE_COUNTERS
#endif
#ifndef GR_CTRLPORT
#define GR_CTRLPORT
#endif
#ifndef GR_RPCSERVER_ENABLED
/* #undef GR_RPCSERVER_ENABLED */
#endif
#ifndef GR_RPCSERVER_THRIFT
/* #undef GR_RPCSERVER_THRIFT */
#endif
#ifndef THRIFT_HAS_VERSION_H
/* #undef THRIFT_HAS_VERSION_H */
#endif
#ifndef THRIFT_HAS_THREADFACTORY_H
/* #undef THRIFT_HAS_THREADFACTORY_H */
#endif
#ifndef GR_MPLIB_GMP
/* #undef GR_MPLIB_GMP */
#endif
#ifndef GR_MPLIB_MPIR
#define GR_MPLIB_MPIR
#endif

#endif /* GNURADIO_CONFIG_H */
