# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _blocks_swig2
else:
    import _blocks_swig2

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def high_res_timer_now() -> "gr::high_res_timer_type":
    r"""high_res_timer_now() -> gr::high_res_timer_type"""
    return _blocks_swig2.high_res_timer_now()

def high_res_timer_now_perfmon() -> "gr::high_res_timer_type":
    r"""high_res_timer_now_perfmon() -> gr::high_res_timer_type"""
    return _blocks_swig2.high_res_timer_now_perfmon()

def high_res_timer_tps() -> "gr::high_res_timer_type":
    r"""high_res_timer_tps() -> gr::high_res_timer_type"""
    return _blocks_swig2.high_res_timer_tps()

def high_res_timer_epoch() -> "gr::high_res_timer_type":
    r"""high_res_timer_epoch() -> gr::high_res_timer_type"""
    return _blocks_swig2.high_res_timer_epoch()
class char_to_float(object):
    r"""Proxy of C++ gr::blocks::char_to_float class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1, scale: "float"=1.0) -> "gr::blocks::char_to_float::sptr":
        r"""make(size_t vlen=1, float scale=1.0) -> char_to_float_sptr"""
        return _blocks_swig2.char_to_float_make(vlen, scale)

    def scale(self) -> "float":
        r"""scale(char_to_float self) -> float"""
        return _blocks_swig2.char_to_float_scale(self)

    def set_scale(self, scale: "float") -> "void":
        r"""set_scale(char_to_float self, float scale)"""
        return _blocks_swig2.char_to_float_set_scale(self, scale)
    __swig_destroy__ = _blocks_swig2.delete_char_to_float

# Register char_to_float in _blocks_swig2:
_blocks_swig2.char_to_float_swigregister(char_to_float)

def char_to_float_make(vlen: "size_t"=1, scale: "float"=1.0) -> "gr::blocks::char_to_float::sptr":
    r"""char_to_float_make(size_t vlen=1, float scale=1.0) -> char_to_float_sptr"""
    return _blocks_swig2.char_to_float_make(vlen, scale)

class bin_statistics_f(object):
    r"""Proxy of C++ gr::blocks::bin_statistics_f class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "unsigned int", msgq: "msg_queue_sptr", tune: "gr::feval_dd *", tune_delay: "size_t", dwell_delay: "size_t") -> "gr::blocks::bin_statistics_f::sptr":
        r"""make(unsigned int vlen, msg_queue_sptr msgq, gr::feval_dd * tune, size_t tune_delay, size_t dwell_delay) -> bin_statistics_f_sptr"""
        return _blocks_swig2.bin_statistics_f_make(vlen, msgq, tune, tune_delay, dwell_delay)
    __swig_destroy__ = _blocks_swig2.delete_bin_statistics_f

# Register bin_statistics_f in _blocks_swig2:
_blocks_swig2.bin_statistics_f_swigregister(bin_statistics_f)

def bin_statistics_f_make(vlen: "unsigned int", msgq: "msg_queue_sptr", tune: "gr::feval_dd *", tune_delay: "size_t", dwell_delay: "size_t") -> "gr::blocks::bin_statistics_f::sptr":
    r"""bin_statistics_f_make(unsigned int vlen, msg_queue_sptr msgq, gr::feval_dd * tune, size_t tune_delay, size_t dwell_delay) -> bin_statistics_f_sptr"""
    return _blocks_swig2.bin_statistics_f_make(vlen, msgq, tune, tune_delay, dwell_delay)

class burst_tagger(object):
    r"""Proxy of C++ gr::blocks::burst_tagger class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(itemsize: "size_t") -> "gr::blocks::burst_tagger::sptr":
        r"""make(size_t itemsize) -> burst_tagger_sptr"""
        return _blocks_swig2.burst_tagger_make(itemsize)

    def set_true_tag(self, key: "std::string const &", value: "bool") -> "void":
        r"""set_true_tag(burst_tagger self, std::string const & key, bool value)"""
        return _blocks_swig2.burst_tagger_set_true_tag(self, key, value)

    def set_false_tag(self, key: "std::string const &", value: "bool") -> "void":
        r"""set_false_tag(burst_tagger self, std::string const & key, bool value)"""
        return _blocks_swig2.burst_tagger_set_false_tag(self, key, value)
    __swig_destroy__ = _blocks_swig2.delete_burst_tagger

# Register burst_tagger in _blocks_swig2:
_blocks_swig2.burst_tagger_swigregister(burst_tagger)

def burst_tagger_make(itemsize: "size_t") -> "gr::blocks::burst_tagger::sptr":
    r"""burst_tagger_make(size_t itemsize) -> burst_tagger_sptr"""
    return _blocks_swig2.burst_tagger_make(itemsize)

class char_to_short(object):
    r"""Proxy of C++ gr::blocks::char_to_short class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::char_to_short::sptr":
        r"""make(size_t vlen=1) -> char_to_short_sptr"""
        return _blocks_swig2.char_to_short_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_char_to_short

# Register char_to_short in _blocks_swig2:
_blocks_swig2.char_to_short_swigregister(char_to_short)

def char_to_short_make(vlen: "size_t"=1) -> "gr::blocks::char_to_short::sptr":
    r"""char_to_short_make(size_t vlen=1) -> char_to_short_sptr"""
    return _blocks_swig2.char_to_short_make(vlen)

class check_lfsr_32k_s(object):
    r"""Proxy of C++ gr::blocks::check_lfsr_32k_s class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make() -> "gr::blocks::check_lfsr_32k_s::sptr":
        r"""make() -> check_lfsr_32k_s_sptr"""
        return _blocks_swig2.check_lfsr_32k_s_make()

    def ntotal(self) -> "long":
        r"""ntotal(check_lfsr_32k_s self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_ntotal(self)

    def nright(self) -> "long":
        r"""nright(check_lfsr_32k_s self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_nright(self)

    def runlength(self) -> "long":
        r"""runlength(check_lfsr_32k_s self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_runlength(self)
    __swig_destroy__ = _blocks_swig2.delete_check_lfsr_32k_s

# Register check_lfsr_32k_s in _blocks_swig2:
_blocks_swig2.check_lfsr_32k_s_swigregister(check_lfsr_32k_s)

def check_lfsr_32k_s_make() -> "gr::blocks::check_lfsr_32k_s::sptr":
    r"""check_lfsr_32k_s_make() -> check_lfsr_32k_s_sptr"""
    return _blocks_swig2.check_lfsr_32k_s_make()

class complex_to_interleaved_char(object):
    r"""Proxy of C++ gr::blocks::complex_to_interleaved_char class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vector: "bool"=False) -> "gr::blocks::complex_to_interleaved_char::sptr":
        r"""make(bool vector=False) -> complex_to_interleaved_char_sptr"""
        return _blocks_swig2.complex_to_interleaved_char_make(vector)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_interleaved_char

# Register complex_to_interleaved_char in _blocks_swig2:
_blocks_swig2.complex_to_interleaved_char_swigregister(complex_to_interleaved_char)

def complex_to_interleaved_char_make(vector: "bool"=False) -> "gr::blocks::complex_to_interleaved_char::sptr":
    r"""complex_to_interleaved_char_make(bool vector=False) -> complex_to_interleaved_char_sptr"""
    return _blocks_swig2.complex_to_interleaved_char_make(vector)

class complex_to_interleaved_short(object):
    r"""Proxy of C++ gr::blocks::complex_to_interleaved_short class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vector: "bool"=False) -> "gr::blocks::complex_to_interleaved_short::sptr":
        r"""make(bool vector=False) -> complex_to_interleaved_short_sptr"""
        return _blocks_swig2.complex_to_interleaved_short_make(vector)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_interleaved_short

# Register complex_to_interleaved_short in _blocks_swig2:
_blocks_swig2.complex_to_interleaved_short_swigregister(complex_to_interleaved_short)

def complex_to_interleaved_short_make(vector: "bool"=False) -> "gr::blocks::complex_to_interleaved_short::sptr":
    r"""complex_to_interleaved_short_make(bool vector=False) -> complex_to_interleaved_short_sptr"""
    return _blocks_swig2.complex_to_interleaved_short_make(vector)

class complex_to_float(object):
    r"""Proxy of C++ gr::blocks::complex_to_float class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_float::sptr":
        r"""make(size_t vlen=1) -> complex_to_float_sptr"""
        return _blocks_swig2.complex_to_float_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_float

# Register complex_to_float in _blocks_swig2:
_blocks_swig2.complex_to_float_swigregister(complex_to_float)

def complex_to_float_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_float::sptr":
    r"""complex_to_float_make(size_t vlen=1) -> complex_to_float_sptr"""
    return _blocks_swig2.complex_to_float_make(vlen)

class complex_to_magphase(object):
    r"""Proxy of C++ gr::blocks::complex_to_magphase class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_magphase::sptr":
        r"""make(size_t vlen=1) -> complex_to_magphase_sptr"""
        return _blocks_swig2.complex_to_magphase_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_magphase

# Register complex_to_magphase in _blocks_swig2:
_blocks_swig2.complex_to_magphase_swigregister(complex_to_magphase)

def complex_to_magphase_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_magphase::sptr":
    r"""complex_to_magphase_make(size_t vlen=1) -> complex_to_magphase_sptr"""
    return _blocks_swig2.complex_to_magphase_make(vlen)

class complex_to_real(object):
    r"""Proxy of C++ gr::blocks::complex_to_real class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_real::sptr":
        r"""make(size_t vlen=1) -> complex_to_real_sptr"""
        return _blocks_swig2.complex_to_real_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_real

# Register complex_to_real in _blocks_swig2:
_blocks_swig2.complex_to_real_swigregister(complex_to_real)

def complex_to_real_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_real::sptr":
    r"""complex_to_real_make(size_t vlen=1) -> complex_to_real_sptr"""
    return _blocks_swig2.complex_to_real_make(vlen)

class complex_to_imag(object):
    r"""Proxy of C++ gr::blocks::complex_to_imag class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_imag::sptr":
        r"""make(size_t vlen=1) -> complex_to_imag_sptr"""
        return _blocks_swig2.complex_to_imag_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_imag

# Register complex_to_imag in _blocks_swig2:
_blocks_swig2.complex_to_imag_swigregister(complex_to_imag)

def complex_to_imag_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_imag::sptr":
    r"""complex_to_imag_make(size_t vlen=1) -> complex_to_imag_sptr"""
    return _blocks_swig2.complex_to_imag_make(vlen)

class complex_to_mag(object):
    r"""Proxy of C++ gr::blocks::complex_to_mag class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_mag::sptr":
        r"""make(size_t vlen=1) -> complex_to_mag_sptr"""
        return _blocks_swig2.complex_to_mag_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_mag

# Register complex_to_mag in _blocks_swig2:
_blocks_swig2.complex_to_mag_swigregister(complex_to_mag)

def complex_to_mag_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_mag::sptr":
    r"""complex_to_mag_make(size_t vlen=1) -> complex_to_mag_sptr"""
    return _blocks_swig2.complex_to_mag_make(vlen)

class complex_to_mag_squared(object):
    r"""Proxy of C++ gr::blocks::complex_to_mag_squared class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_mag_squared::sptr":
        r"""make(size_t vlen=1) -> complex_to_mag_squared_sptr"""
        return _blocks_swig2.complex_to_mag_squared_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_mag_squared

# Register complex_to_mag_squared in _blocks_swig2:
_blocks_swig2.complex_to_mag_squared_swigregister(complex_to_mag_squared)

def complex_to_mag_squared_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_mag_squared::sptr":
    r"""complex_to_mag_squared_make(size_t vlen=1) -> complex_to_mag_squared_sptr"""
    return _blocks_swig2.complex_to_mag_squared_make(vlen)

class complex_to_arg(object):
    r"""Proxy of C++ gr::blocks::complex_to_arg class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t"=1) -> "gr::blocks::complex_to_arg::sptr":
        r"""make(size_t vlen=1) -> complex_to_arg_sptr"""
        return _blocks_swig2.complex_to_arg_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_arg

# Register complex_to_arg in _blocks_swig2:
_blocks_swig2.complex_to_arg_swigregister(complex_to_arg)

def complex_to_arg_make(vlen: "size_t"=1) -> "gr::blocks::complex_to_arg::sptr":
    r"""complex_to_arg_make(size_t vlen=1) -> complex_to_arg_sptr"""
    return _blocks_swig2.complex_to_arg_make(vlen)

class conjugate_cc(object):
    r"""Proxy of C++ gr::blocks::conjugate_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make() -> "gr::blocks::conjugate_cc::sptr":
        r"""make() -> conjugate_cc_sptr"""
        return _blocks_swig2.conjugate_cc_make()
    __swig_destroy__ = _blocks_swig2.delete_conjugate_cc

# Register conjugate_cc in _blocks_swig2:
_blocks_swig2.conjugate_cc_swigregister(conjugate_cc)

def conjugate_cc_make() -> "gr::blocks::conjugate_cc::sptr":
    r"""conjugate_cc_make() -> conjugate_cc_sptr"""
    return _blocks_swig2.conjugate_cc_make()

class and_const_bb(object):
    r"""Proxy of C++ gr::blocks::and_const< std::uint8_t > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(k: "std::uint8_t") -> "gr::blocks::and_const< unsigned char >::sptr":
        r"""make(std::uint8_t k) -> and_const_bb_sptr"""
        return _blocks_swig2.and_const_bb_make(k)

    def k(self) -> "std::uint8_t":
        r"""k(and_const_bb self) -> std::uint8_t"""
        return _blocks_swig2.and_const_bb_k(self)

    def set_k(self, k: "std::uint8_t") -> "void":
        r"""set_k(and_const_bb self, std::uint8_t k)"""
        return _blocks_swig2.and_const_bb_set_k(self, k)
    __swig_destroy__ = _blocks_swig2.delete_and_const_bb

# Register and_const_bb in _blocks_swig2:
_blocks_swig2.and_const_bb_swigregister(and_const_bb)

def and_const_bb_make(k: "std::uint8_t") -> "gr::blocks::and_const< unsigned char >::sptr":
    r"""and_const_bb_make(std::uint8_t k) -> and_const_bb_sptr"""
    return _blocks_swig2.and_const_bb_make(k)

class and_const_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::and_const< std::uint8_t > > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(and_const_bb_sptr self) -> and_const_bb_sptr
        __init__(and_const_bb_sptr self, and_const_bb p) -> and_const_bb_sptr
        """
        _blocks_swig2.and_const_bb_sptr_swiginit(self, _blocks_swig2.new_and_const_bb_sptr(*args))

    def __deref__(self) -> "gr::blocks::and_const< std::uint8_t > *":
        r"""__deref__(and_const_bb_sptr self) -> and_const_bb"""
        return _blocks_swig2.and_const_bb_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_and_const_bb_sptr

    def make(self, k: "std::uint8_t") -> "gr::blocks::and_const< unsigned char >::sptr":
        r"""make(and_const_bb_sptr self, std::uint8_t k) -> and_const_bb_sptr"""
        return _blocks_swig2.and_const_bb_sptr_make(self, k)

    def k(self) -> "std::uint8_t":
        r"""k(and_const_bb_sptr self) -> std::uint8_t"""
        return _blocks_swig2.and_const_bb_sptr_k(self)

    def set_k(self, k: "std::uint8_t") -> "void":
        r"""set_k(and_const_bb_sptr self, std::uint8_t k)"""
        return _blocks_swig2.and_const_bb_sptr_set_k(self, k)

    def history(self) -> "unsigned int":
        r"""history(and_const_bb_sptr self) -> unsigned int"""
        return _blocks_swig2.and_const_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(and_const_bb_sptr self, int which, int delay)
        declare_sample_delay(and_const_bb_sptr self, unsigned int delay)
        """
        return _blocks_swig2.and_const_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(and_const_bb_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.and_const_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(and_const_bb_sptr self, int multiple)"""
        return _blocks_swig2.and_const_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(and_const_bb_sptr self) -> int"""
        return _blocks_swig2.and_const_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(and_const_bb_sptr self) -> double"""
        return _blocks_swig2.and_const_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(and_const_bb_sptr self) -> uint64_t"""
        return _blocks_swig2.and_const_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(and_const_bb_sptr self) -> uint64_t"""
        return _blocks_swig2.and_const_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(and_const_bb_sptr self) -> bool"""
        return _blocks_swig2.and_const_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(and_const_bb_sptr self) -> bool"""
        return _blocks_swig2.and_const_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(and_const_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.and_const_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(and_const_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.and_const_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(and_const_bb_sptr self, std::string level)"""
        return _blocks_swig2.and_const_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(and_const_bb_sptr self) -> std::string"""
        return _blocks_swig2.and_const_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(and_const_bb_sptr self) -> int"""
        return _blocks_swig2.and_const_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(and_const_bb_sptr self, int m)"""
        return _blocks_swig2.and_const_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(and_const_bb_sptr self)"""
        return _blocks_swig2.and_const_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(and_const_bb_sptr self) -> bool"""
        return _blocks_swig2.and_const_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(and_const_bb_sptr self, int m)"""
        return _blocks_swig2.and_const_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(and_const_bb_sptr self) -> int"""
        return _blocks_swig2.and_const_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(and_const_bb_sptr self, int i) -> long"""
        return _blocks_swig2.and_const_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(and_const_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(and_const_bb_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.and_const_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(and_const_bb_sptr self, int i) -> long"""
        return _blocks_swig2.and_const_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(and_const_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(and_const_bb_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.and_const_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(and_const_bb_sptr self, int which) -> float
        pc_input_buffers_full(and_const_bb_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(and_const_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(and_const_bb_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(and_const_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(and_const_bb_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(and_const_bb_sptr self, int which) -> float
        pc_output_buffers_full(and_const_bb_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(and_const_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(and_const_bb_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(and_const_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(and_const_bb_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(and_const_bb_sptr self) -> float"""
        return _blocks_swig2.and_const_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(and_const_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.and_const_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(and_const_bb_sptr self)"""
        return _blocks_swig2.and_const_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(and_const_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.and_const_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(and_const_bb_sptr self) -> int"""
        return _blocks_swig2.and_const_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(and_const_bb_sptr self) -> int"""
        return _blocks_swig2.and_const_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(and_const_bb_sptr self, int priority) -> int"""
        return _blocks_swig2.and_const_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(and_const_bb_sptr self) -> std::string"""
        return _blocks_swig2.and_const_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(and_const_bb_sptr self) -> std::string"""
        return _blocks_swig2.and_const_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(and_const_bb_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.and_const_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(and_const_bb_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.and_const_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(and_const_bb_sptr self) -> long"""
        return _blocks_swig2.and_const_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(and_const_bb_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.and_const_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(and_const_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.and_const_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(and_const_bb_sptr self) -> std::string"""
        return _blocks_swig2.and_const_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(and_const_bb_sptr self, std::string name)"""
        return _blocks_swig2.and_const_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(and_const_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.and_const_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(and_const_bb_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(and_const_bb_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(and_const_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_bb_sptr_message_subscribers(self, which_port)

# Register and_const_bb_sptr in _blocks_swig2:
_blocks_swig2.and_const_bb_sptr_swigregister(and_const_bb_sptr)


and_const_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
and_const_bb = and_const_bb.make

class and_const_ss(object):
    r"""Proxy of C++ gr::blocks::and_const< std::int16_t > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(k: "std::int16_t") -> "gr::blocks::and_const< short >::sptr":
        r"""make(std::int16_t k) -> and_const_ss_sptr"""
        return _blocks_swig2.and_const_ss_make(k)

    def k(self) -> "std::int16_t":
        r"""k(and_const_ss self) -> std::int16_t"""
        return _blocks_swig2.and_const_ss_k(self)

    def set_k(self, k: "std::int16_t") -> "void":
        r"""set_k(and_const_ss self, std::int16_t k)"""
        return _blocks_swig2.and_const_ss_set_k(self, k)
    __swig_destroy__ = _blocks_swig2.delete_and_const_ss

# Register and_const_ss in _blocks_swig2:
_blocks_swig2.and_const_ss_swigregister(and_const_ss)

def and_const_ss_make(k: "std::int16_t") -> "gr::blocks::and_const< short >::sptr":
    r"""and_const_ss_make(std::int16_t k) -> and_const_ss_sptr"""
    return _blocks_swig2.and_const_ss_make(k)

class and_const_ss_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::and_const< std::int16_t > > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(and_const_ss_sptr self) -> and_const_ss_sptr
        __init__(and_const_ss_sptr self, and_const_ss p) -> and_const_ss_sptr
        """
        _blocks_swig2.and_const_ss_sptr_swiginit(self, _blocks_swig2.new_and_const_ss_sptr(*args))

    def __deref__(self) -> "gr::blocks::and_const< std::int16_t > *":
        r"""__deref__(and_const_ss_sptr self) -> and_const_ss"""
        return _blocks_swig2.and_const_ss_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_and_const_ss_sptr

    def make(self, k: "std::int16_t") -> "gr::blocks::and_const< short >::sptr":
        r"""make(and_const_ss_sptr self, std::int16_t k) -> and_const_ss_sptr"""
        return _blocks_swig2.and_const_ss_sptr_make(self, k)

    def k(self) -> "std::int16_t":
        r"""k(and_const_ss_sptr self) -> std::int16_t"""
        return _blocks_swig2.and_const_ss_sptr_k(self)

    def set_k(self, k: "std::int16_t") -> "void":
        r"""set_k(and_const_ss_sptr self, std::int16_t k)"""
        return _blocks_swig2.and_const_ss_sptr_set_k(self, k)

    def history(self) -> "unsigned int":
        r"""history(and_const_ss_sptr self) -> unsigned int"""
        return _blocks_swig2.and_const_ss_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(and_const_ss_sptr self, int which, int delay)
        declare_sample_delay(and_const_ss_sptr self, unsigned int delay)
        """
        return _blocks_swig2.and_const_ss_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(and_const_ss_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.and_const_ss_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(and_const_ss_sptr self, int multiple)"""
        return _blocks_swig2.and_const_ss_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(and_const_ss_sptr self) -> int"""
        return _blocks_swig2.and_const_ss_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(and_const_ss_sptr self) -> double"""
        return _blocks_swig2.and_const_ss_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(and_const_ss_sptr self) -> uint64_t"""
        return _blocks_swig2.and_const_ss_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(and_const_ss_sptr self) -> uint64_t"""
        return _blocks_swig2.and_const_ss_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(and_const_ss_sptr self) -> bool"""
        return _blocks_swig2.and_const_ss_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(and_const_ss_sptr self) -> bool"""
        return _blocks_swig2.and_const_ss_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(and_const_ss_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.and_const_ss_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(and_const_ss_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.and_const_ss_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(and_const_ss_sptr self, std::string level)"""
        return _blocks_swig2.and_const_ss_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(and_const_ss_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ss_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(and_const_ss_sptr self) -> int"""
        return _blocks_swig2.and_const_ss_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(and_const_ss_sptr self, int m)"""
        return _blocks_swig2.and_const_ss_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(and_const_ss_sptr self)"""
        return _blocks_swig2.and_const_ss_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(and_const_ss_sptr self) -> bool"""
        return _blocks_swig2.and_const_ss_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(and_const_ss_sptr self, int m)"""
        return _blocks_swig2.and_const_ss_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(and_const_ss_sptr self) -> int"""
        return _blocks_swig2.and_const_ss_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(and_const_ss_sptr self, int i) -> long"""
        return _blocks_swig2.and_const_ss_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(and_const_ss_sptr self, long max_output_buffer)
        set_max_output_buffer(and_const_ss_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.and_const_ss_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(and_const_ss_sptr self, int i) -> long"""
        return _blocks_swig2.and_const_ss_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(and_const_ss_sptr self, long min_output_buffer)
        set_min_output_buffer(and_const_ss_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.and_const_ss_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(and_const_ss_sptr self, int which) -> float
        pc_input_buffers_full(and_const_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ss_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(and_const_ss_sptr self, int which) -> float
        pc_input_buffers_full_avg(and_const_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ss_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(and_const_ss_sptr self, int which) -> float
        pc_input_buffers_full_var(and_const_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ss_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(and_const_ss_sptr self, int which) -> float
        pc_output_buffers_full(and_const_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ss_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(and_const_ss_sptr self, int which) -> float
        pc_output_buffers_full_avg(and_const_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ss_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(and_const_ss_sptr self, int which) -> float
        pc_output_buffers_full_var(and_const_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ss_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(and_const_ss_sptr self) -> float"""
        return _blocks_swig2.and_const_ss_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(and_const_ss_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.and_const_ss_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(and_const_ss_sptr self)"""
        return _blocks_swig2.and_const_ss_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(and_const_ss_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.and_const_ss_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(and_const_ss_sptr self) -> int"""
        return _blocks_swig2.and_const_ss_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(and_const_ss_sptr self) -> int"""
        return _blocks_swig2.and_const_ss_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(and_const_ss_sptr self, int priority) -> int"""
        return _blocks_swig2.and_const_ss_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(and_const_ss_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ss_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(and_const_ss_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ss_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(and_const_ss_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.and_const_ss_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(and_const_ss_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.and_const_ss_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(and_const_ss_sptr self) -> long"""
        return _blocks_swig2.and_const_ss_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(and_const_ss_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.and_const_ss_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(and_const_ss_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.and_const_ss_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(and_const_ss_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ss_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(and_const_ss_sptr self, std::string name)"""
        return _blocks_swig2.and_const_ss_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(and_const_ss_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.and_const_ss_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(and_const_ss_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_ss_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(and_const_ss_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_ss_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(and_const_ss_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_ss_sptr_message_subscribers(self, which_port)

# Register and_const_ss_sptr in _blocks_swig2:
_blocks_swig2.and_const_ss_sptr_swigregister(and_const_ss_sptr)


and_const_ss_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
and_const_ss = and_const_ss.make

class and_const_ii(object):
    r"""Proxy of C++ gr::blocks::and_const< std::int32_t > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def make(k: "std::int32_t") -> "gr::blocks::and_const< int >::sptr":
        r"""make(std::int32_t k) -> and_const_ii_sptr"""
        return _blocks_swig2.and_const_ii_make(k)

    def k(self) -> "std::int32_t":
        r"""k(and_const_ii self) -> std::int32_t"""
        return _blocks_swig2.and_const_ii_k(self)

    def set_k(self, k: "std::int32_t") -> "void":
        r"""set_k(and_const_ii self, std::int32_t k)"""
        return _blocks_swig2.and_const_ii_set_k(self, k)
    __swig_destroy__ = _blocks_swig2.delete_and_const_ii

# Register and_const_ii in _blocks_swig2:
_blocks_swig2.and_const_ii_swigregister(and_const_ii)

def and_const_ii_make(k: "std::int32_t") -> "gr::blocks::and_const< int >::sptr":
    r"""and_const_ii_make(std::int32_t k) -> and_const_ii_sptr"""
    return _blocks_swig2.and_const_ii_make(k)

class and_const_ii_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::and_const< std::int32_t > > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(and_const_ii_sptr self) -> and_const_ii_sptr
        __init__(and_const_ii_sptr self, and_const_ii p) -> and_const_ii_sptr
        """
        _blocks_swig2.and_const_ii_sptr_swiginit(self, _blocks_swig2.new_and_const_ii_sptr(*args))

    def __deref__(self) -> "gr::blocks::and_const< std::int32_t > *":
        r"""__deref__(and_const_ii_sptr self) -> and_const_ii"""
        return _blocks_swig2.and_const_ii_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_and_const_ii_sptr

    def make(self, k: "std::int32_t") -> "gr::blocks::and_const< int >::sptr":
        r"""make(and_const_ii_sptr self, std::int32_t k) -> and_const_ii_sptr"""
        return _blocks_swig2.and_const_ii_sptr_make(self, k)

    def k(self) -> "std::int32_t":
        r"""k(and_const_ii_sptr self) -> std::int32_t"""
        return _blocks_swig2.and_const_ii_sptr_k(self)

    def set_k(self, k: "std::int32_t") -> "void":
        r"""set_k(and_const_ii_sptr self, std::int32_t k)"""
        return _blocks_swig2.and_const_ii_sptr_set_k(self, k)

    def history(self) -> "unsigned int":
        r"""history(and_const_ii_sptr self) -> unsigned int"""
        return _blocks_swig2.and_const_ii_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(and_const_ii_sptr self, int which, int delay)
        declare_sample_delay(and_const_ii_sptr self, unsigned int delay)
        """
        return _blocks_swig2.and_const_ii_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(and_const_ii_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.and_const_ii_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(and_const_ii_sptr self, int multiple)"""
        return _blocks_swig2.and_const_ii_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(and_const_ii_sptr self) -> int"""
        return _blocks_swig2.and_const_ii_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(and_const_ii_sptr self) -> double"""
        return _blocks_swig2.and_const_ii_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(and_const_ii_sptr self) -> uint64_t"""
        return _blocks_swig2.and_const_ii_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(and_const_ii_sptr self) -> uint64_t"""
        return _blocks_swig2.and_const_ii_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(and_const_ii_sptr self) -> bool"""
        return _blocks_swig2.and_const_ii_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(and_const_ii_sptr self) -> bool"""
        return _blocks_swig2.and_const_ii_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(and_const_ii_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.and_const_ii_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(and_const_ii_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.and_const_ii_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(and_const_ii_sptr self, std::string level)"""
        return _blocks_swig2.and_const_ii_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(and_const_ii_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ii_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(and_const_ii_sptr self) -> int"""
        return _blocks_swig2.and_const_ii_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(and_const_ii_sptr self, int m)"""
        return _blocks_swig2.and_const_ii_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(and_const_ii_sptr self)"""
        return _blocks_swig2.and_const_ii_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(and_const_ii_sptr self) -> bool"""
        return _blocks_swig2.and_const_ii_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(and_const_ii_sptr self, int m)"""
        return _blocks_swig2.and_const_ii_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(and_const_ii_sptr self) -> int"""
        return _blocks_swig2.and_const_ii_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(and_const_ii_sptr self, int i) -> long"""
        return _blocks_swig2.and_const_ii_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(and_const_ii_sptr self, long max_output_buffer)
        set_max_output_buffer(and_const_ii_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.and_const_ii_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(and_const_ii_sptr self, int i) -> long"""
        return _blocks_swig2.and_const_ii_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(and_const_ii_sptr self, long min_output_buffer)
        set_min_output_buffer(and_const_ii_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.and_const_ii_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(and_const_ii_sptr self, int which) -> float
        pc_input_buffers_full(and_const_ii_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ii_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(and_const_ii_sptr self, int which) -> float
        pc_input_buffers_full_avg(and_const_ii_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ii_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(and_const_ii_sptr self, int which) -> float
        pc_input_buffers_full_var(and_const_ii_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ii_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(and_const_ii_sptr self, int which) -> float
        pc_output_buffers_full(and_const_ii_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ii_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(and_const_ii_sptr self, int which) -> float
        pc_output_buffers_full_avg(and_const_ii_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ii_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(and_const_ii_sptr self, int which) -> float
        pc_output_buffers_full_var(and_const_ii_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.and_const_ii_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(and_const_ii_sptr self) -> float"""
        return _blocks_swig2.and_const_ii_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(and_const_ii_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.and_const_ii_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(and_const_ii_sptr self)"""
        return _blocks_swig2.and_const_ii_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(and_const_ii_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.and_const_ii_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(and_const_ii_sptr self) -> int"""
        return _blocks_swig2.and_const_ii_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(and_const_ii_sptr self) -> int"""
        return _blocks_swig2.and_const_ii_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(and_const_ii_sptr self, int priority) -> int"""
        return _blocks_swig2.and_const_ii_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(and_const_ii_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ii_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(and_const_ii_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ii_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(and_const_ii_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.and_const_ii_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(and_const_ii_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.and_const_ii_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(and_const_ii_sptr self) -> long"""
        return _blocks_swig2.and_const_ii_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(and_const_ii_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.and_const_ii_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(and_const_ii_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.and_const_ii_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(and_const_ii_sptr self) -> std::string"""
        return _blocks_swig2.and_const_ii_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(and_const_ii_sptr self, std::string name)"""
        return _blocks_swig2.and_const_ii_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(and_const_ii_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.and_const_ii_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(and_const_ii_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_ii_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(and_const_ii_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_ii_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(and_const_ii_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.and_const_ii_sptr_message_subscribers(self, which_port)

# Register and_const_ii_sptr in _blocks_swig2:
_blocks_swig2.and_const_ii_sptr_swigregister(and_const_ii_sptr)


and_const_ii_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
and_const_ii = and_const_ii.make

class argmax_fs(object):
    r"""Proxy of C++ gr::blocks::argmax< float > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t") -> "gr::blocks::argmax< float >::sptr":
        r"""make(size_t vlen) -> argmax_fs_sptr"""
        return _blocks_swig2.argmax_fs_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_argmax_fs

# Register argmax_fs in _blocks_swig2:
_blocks_swig2.argmax_fs_swigregister(argmax_fs)

def argmax_fs_make(vlen: "size_t") -> "gr::blocks::argmax< float >::sptr":
    r"""argmax_fs_make(size_t vlen) -> argmax_fs_sptr"""
    return _blocks_swig2.argmax_fs_make(vlen)

class argmax_fs_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::argmax< float > > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(argmax_fs_sptr self) -> argmax_fs_sptr
        __init__(argmax_fs_sptr self, argmax_fs p) -> argmax_fs_sptr
        """
        _blocks_swig2.argmax_fs_sptr_swiginit(self, _blocks_swig2.new_argmax_fs_sptr(*args))

    def __deref__(self) -> "gr::blocks::argmax< float > *":
        r"""__deref__(argmax_fs_sptr self) -> argmax_fs"""
        return _blocks_swig2.argmax_fs_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_argmax_fs_sptr

    def make(self, vlen: "size_t") -> "gr::blocks::argmax< float >::sptr":
        r"""make(argmax_fs_sptr self, size_t vlen) -> argmax_fs_sptr"""
        return _blocks_swig2.argmax_fs_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(argmax_fs_sptr self) -> unsigned int"""
        return _blocks_swig2.argmax_fs_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(argmax_fs_sptr self, int which, int delay)
        declare_sample_delay(argmax_fs_sptr self, unsigned int delay)
        """
        return _blocks_swig2.argmax_fs_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(argmax_fs_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.argmax_fs_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(argmax_fs_sptr self, int multiple)"""
        return _blocks_swig2.argmax_fs_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(argmax_fs_sptr self) -> int"""
        return _blocks_swig2.argmax_fs_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(argmax_fs_sptr self) -> double"""
        return _blocks_swig2.argmax_fs_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(argmax_fs_sptr self) -> uint64_t"""
        return _blocks_swig2.argmax_fs_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(argmax_fs_sptr self) -> uint64_t"""
        return _blocks_swig2.argmax_fs_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(argmax_fs_sptr self) -> bool"""
        return _blocks_swig2.argmax_fs_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(argmax_fs_sptr self) -> bool"""
        return _blocks_swig2.argmax_fs_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(argmax_fs_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.argmax_fs_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(argmax_fs_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.argmax_fs_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(argmax_fs_sptr self, std::string level)"""
        return _blocks_swig2.argmax_fs_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(argmax_fs_sptr self) -> std::string"""
        return _blocks_swig2.argmax_fs_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(argmax_fs_sptr self) -> int"""
        return _blocks_swig2.argmax_fs_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(argmax_fs_sptr self, int m)"""
        return _blocks_swig2.argmax_fs_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(argmax_fs_sptr self)"""
        return _blocks_swig2.argmax_fs_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(argmax_fs_sptr self) -> bool"""
        return _blocks_swig2.argmax_fs_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(argmax_fs_sptr self, int m)"""
        return _blocks_swig2.argmax_fs_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(argmax_fs_sptr self) -> int"""
        return _blocks_swig2.argmax_fs_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(argmax_fs_sptr self, int i) -> long"""
        return _blocks_swig2.argmax_fs_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(argmax_fs_sptr self, long max_output_buffer)
        set_max_output_buffer(argmax_fs_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.argmax_fs_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(argmax_fs_sptr self, int i) -> long"""
        return _blocks_swig2.argmax_fs_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(argmax_fs_sptr self, long min_output_buffer)
        set_min_output_buffer(argmax_fs_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.argmax_fs_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(argmax_fs_sptr self, int which) -> float
        pc_input_buffers_full(argmax_fs_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_fs_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(argmax_fs_sptr self, int which) -> float
        pc_input_buffers_full_avg(argmax_fs_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_fs_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(argmax_fs_sptr self, int which) -> float
        pc_input_buffers_full_var(argmax_fs_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_fs_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(argmax_fs_sptr self, int which) -> float
        pc_output_buffers_full(argmax_fs_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_fs_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(argmax_fs_sptr self, int which) -> float
        pc_output_buffers_full_avg(argmax_fs_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_fs_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(argmax_fs_sptr self, int which) -> float
        pc_output_buffers_full_var(argmax_fs_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_fs_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(argmax_fs_sptr self) -> float"""
        return _blocks_swig2.argmax_fs_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(argmax_fs_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.argmax_fs_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(argmax_fs_sptr self)"""
        return _blocks_swig2.argmax_fs_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(argmax_fs_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.argmax_fs_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(argmax_fs_sptr self) -> int"""
        return _blocks_swig2.argmax_fs_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(argmax_fs_sptr self) -> int"""
        return _blocks_swig2.argmax_fs_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(argmax_fs_sptr self, int priority) -> int"""
        return _blocks_swig2.argmax_fs_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(argmax_fs_sptr self) -> std::string"""
        return _blocks_swig2.argmax_fs_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(argmax_fs_sptr self) -> std::string"""
        return _blocks_swig2.argmax_fs_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(argmax_fs_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.argmax_fs_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(argmax_fs_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.argmax_fs_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(argmax_fs_sptr self) -> long"""
        return _blocks_swig2.argmax_fs_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(argmax_fs_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.argmax_fs_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(argmax_fs_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.argmax_fs_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(argmax_fs_sptr self) -> std::string"""
        return _blocks_swig2.argmax_fs_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(argmax_fs_sptr self, std::string name)"""
        return _blocks_swig2.argmax_fs_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(argmax_fs_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.argmax_fs_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(argmax_fs_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_fs_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(argmax_fs_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_fs_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(argmax_fs_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_fs_sptr_message_subscribers(self, which_port)

# Register argmax_fs_sptr in _blocks_swig2:
_blocks_swig2.argmax_fs_sptr_swigregister(argmax_fs_sptr)


argmax_fs_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
argmax_fs = argmax_fs.make

class argmax_is(object):
    r"""Proxy of C++ gr::blocks::argmax< std::int32_t > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t") -> "gr::blocks::argmax< int >::sptr":
        r"""make(size_t vlen) -> argmax_is_sptr"""
        return _blocks_swig2.argmax_is_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_argmax_is

# Register argmax_is in _blocks_swig2:
_blocks_swig2.argmax_is_swigregister(argmax_is)

def argmax_is_make(vlen: "size_t") -> "gr::blocks::argmax< int >::sptr":
    r"""argmax_is_make(size_t vlen) -> argmax_is_sptr"""
    return _blocks_swig2.argmax_is_make(vlen)

class argmax_is_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::argmax< std::int32_t > > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(argmax_is_sptr self) -> argmax_is_sptr
        __init__(argmax_is_sptr self, argmax_is p) -> argmax_is_sptr
        """
        _blocks_swig2.argmax_is_sptr_swiginit(self, _blocks_swig2.new_argmax_is_sptr(*args))

    def __deref__(self) -> "gr::blocks::argmax< std::int32_t > *":
        r"""__deref__(argmax_is_sptr self) -> argmax_is"""
        return _blocks_swig2.argmax_is_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_argmax_is_sptr

    def make(self, vlen: "size_t") -> "gr::blocks::argmax< int >::sptr":
        r"""make(argmax_is_sptr self, size_t vlen) -> argmax_is_sptr"""
        return _blocks_swig2.argmax_is_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(argmax_is_sptr self) -> unsigned int"""
        return _blocks_swig2.argmax_is_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(argmax_is_sptr self, int which, int delay)
        declare_sample_delay(argmax_is_sptr self, unsigned int delay)
        """
        return _blocks_swig2.argmax_is_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(argmax_is_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.argmax_is_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(argmax_is_sptr self, int multiple)"""
        return _blocks_swig2.argmax_is_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(argmax_is_sptr self) -> int"""
        return _blocks_swig2.argmax_is_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(argmax_is_sptr self) -> double"""
        return _blocks_swig2.argmax_is_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(argmax_is_sptr self) -> uint64_t"""
        return _blocks_swig2.argmax_is_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(argmax_is_sptr self) -> uint64_t"""
        return _blocks_swig2.argmax_is_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(argmax_is_sptr self) -> bool"""
        return _blocks_swig2.argmax_is_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(argmax_is_sptr self) -> bool"""
        return _blocks_swig2.argmax_is_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(argmax_is_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.argmax_is_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(argmax_is_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.argmax_is_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(argmax_is_sptr self, std::string level)"""
        return _blocks_swig2.argmax_is_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(argmax_is_sptr self) -> std::string"""
        return _blocks_swig2.argmax_is_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(argmax_is_sptr self) -> int"""
        return _blocks_swig2.argmax_is_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(argmax_is_sptr self, int m)"""
        return _blocks_swig2.argmax_is_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(argmax_is_sptr self)"""
        return _blocks_swig2.argmax_is_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(argmax_is_sptr self) -> bool"""
        return _blocks_swig2.argmax_is_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(argmax_is_sptr self, int m)"""
        return _blocks_swig2.argmax_is_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(argmax_is_sptr self) -> int"""
        return _blocks_swig2.argmax_is_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(argmax_is_sptr self, int i) -> long"""
        return _blocks_swig2.argmax_is_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(argmax_is_sptr self, long max_output_buffer)
        set_max_output_buffer(argmax_is_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.argmax_is_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(argmax_is_sptr self, int i) -> long"""
        return _blocks_swig2.argmax_is_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(argmax_is_sptr self, long min_output_buffer)
        set_min_output_buffer(argmax_is_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.argmax_is_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(argmax_is_sptr self, int which) -> float
        pc_input_buffers_full(argmax_is_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_is_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(argmax_is_sptr self, int which) -> float
        pc_input_buffers_full_avg(argmax_is_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_is_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(argmax_is_sptr self, int which) -> float
        pc_input_buffers_full_var(argmax_is_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_is_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(argmax_is_sptr self, int which) -> float
        pc_output_buffers_full(argmax_is_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_is_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(argmax_is_sptr self, int which) -> float
        pc_output_buffers_full_avg(argmax_is_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_is_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(argmax_is_sptr self, int which) -> float
        pc_output_buffers_full_var(argmax_is_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_is_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(argmax_is_sptr self) -> float"""
        return _blocks_swig2.argmax_is_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(argmax_is_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.argmax_is_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(argmax_is_sptr self)"""
        return _blocks_swig2.argmax_is_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(argmax_is_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.argmax_is_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(argmax_is_sptr self) -> int"""
        return _blocks_swig2.argmax_is_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(argmax_is_sptr self) -> int"""
        return _blocks_swig2.argmax_is_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(argmax_is_sptr self, int priority) -> int"""
        return _blocks_swig2.argmax_is_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(argmax_is_sptr self) -> std::string"""
        return _blocks_swig2.argmax_is_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(argmax_is_sptr self) -> std::string"""
        return _blocks_swig2.argmax_is_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(argmax_is_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.argmax_is_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(argmax_is_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.argmax_is_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(argmax_is_sptr self) -> long"""
        return _blocks_swig2.argmax_is_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(argmax_is_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.argmax_is_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(argmax_is_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.argmax_is_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(argmax_is_sptr self) -> std::string"""
        return _blocks_swig2.argmax_is_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(argmax_is_sptr self, std::string name)"""
        return _blocks_swig2.argmax_is_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(argmax_is_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.argmax_is_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(argmax_is_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_is_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(argmax_is_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_is_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(argmax_is_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_is_sptr_message_subscribers(self, which_port)

# Register argmax_is_sptr in _blocks_swig2:
_blocks_swig2.argmax_is_sptr_swigregister(argmax_is_sptr)


argmax_is_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
argmax_is = argmax_is.make

class argmax_ss(object):
    r"""Proxy of C++ gr::blocks::argmax< std::int16_t > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(vlen: "size_t") -> "gr::blocks::argmax< short >::sptr":
        r"""make(size_t vlen) -> argmax_ss_sptr"""
        return _blocks_swig2.argmax_ss_make(vlen)
    __swig_destroy__ = _blocks_swig2.delete_argmax_ss

# Register argmax_ss in _blocks_swig2:
_blocks_swig2.argmax_ss_swigregister(argmax_ss)

def argmax_ss_make(vlen: "size_t") -> "gr::blocks::argmax< short >::sptr":
    r"""argmax_ss_make(size_t vlen) -> argmax_ss_sptr"""
    return _blocks_swig2.argmax_ss_make(vlen)

class argmax_ss_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::argmax< std::int16_t > > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(argmax_ss_sptr self) -> argmax_ss_sptr
        __init__(argmax_ss_sptr self, argmax_ss p) -> argmax_ss_sptr
        """
        _blocks_swig2.argmax_ss_sptr_swiginit(self, _blocks_swig2.new_argmax_ss_sptr(*args))

    def __deref__(self) -> "gr::blocks::argmax< std::int16_t > *":
        r"""__deref__(argmax_ss_sptr self) -> argmax_ss"""
        return _blocks_swig2.argmax_ss_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_argmax_ss_sptr

    def make(self, vlen: "size_t") -> "gr::blocks::argmax< short >::sptr":
        r"""make(argmax_ss_sptr self, size_t vlen) -> argmax_ss_sptr"""
        return _blocks_swig2.argmax_ss_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(argmax_ss_sptr self) -> unsigned int"""
        return _blocks_swig2.argmax_ss_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(argmax_ss_sptr self, int which, int delay)
        declare_sample_delay(argmax_ss_sptr self, unsigned int delay)
        """
        return _blocks_swig2.argmax_ss_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(argmax_ss_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.argmax_ss_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(argmax_ss_sptr self, int multiple)"""
        return _blocks_swig2.argmax_ss_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(argmax_ss_sptr self) -> int"""
        return _blocks_swig2.argmax_ss_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(argmax_ss_sptr self) -> double"""
        return _blocks_swig2.argmax_ss_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(argmax_ss_sptr self) -> uint64_t"""
        return _blocks_swig2.argmax_ss_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(argmax_ss_sptr self) -> uint64_t"""
        return _blocks_swig2.argmax_ss_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(argmax_ss_sptr self) -> bool"""
        return _blocks_swig2.argmax_ss_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(argmax_ss_sptr self) -> bool"""
        return _blocks_swig2.argmax_ss_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(argmax_ss_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.argmax_ss_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(argmax_ss_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.argmax_ss_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(argmax_ss_sptr self, std::string level)"""
        return _blocks_swig2.argmax_ss_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(argmax_ss_sptr self) -> std::string"""
        return _blocks_swig2.argmax_ss_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(argmax_ss_sptr self) -> int"""
        return _blocks_swig2.argmax_ss_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(argmax_ss_sptr self, int m)"""
        return _blocks_swig2.argmax_ss_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(argmax_ss_sptr self)"""
        return _blocks_swig2.argmax_ss_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(argmax_ss_sptr self) -> bool"""
        return _blocks_swig2.argmax_ss_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(argmax_ss_sptr self, int m)"""
        return _blocks_swig2.argmax_ss_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(argmax_ss_sptr self) -> int"""
        return _blocks_swig2.argmax_ss_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(argmax_ss_sptr self, int i) -> long"""
        return _blocks_swig2.argmax_ss_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(argmax_ss_sptr self, long max_output_buffer)
        set_max_output_buffer(argmax_ss_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.argmax_ss_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(argmax_ss_sptr self, int i) -> long"""
        return _blocks_swig2.argmax_ss_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(argmax_ss_sptr self, long min_output_buffer)
        set_min_output_buffer(argmax_ss_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.argmax_ss_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(argmax_ss_sptr self, int which) -> float
        pc_input_buffers_full(argmax_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_ss_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(argmax_ss_sptr self, int which) -> float
        pc_input_buffers_full_avg(argmax_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_ss_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(argmax_ss_sptr self, int which) -> float
        pc_input_buffers_full_var(argmax_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_ss_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(argmax_ss_sptr self, int which) -> float
        pc_output_buffers_full(argmax_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_ss_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(argmax_ss_sptr self, int which) -> float
        pc_output_buffers_full_avg(argmax_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_ss_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(argmax_ss_sptr self, int which) -> float
        pc_output_buffers_full_var(argmax_ss_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.argmax_ss_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(argmax_ss_sptr self) -> float"""
        return _blocks_swig2.argmax_ss_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(argmax_ss_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.argmax_ss_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(argmax_ss_sptr self)"""
        return _blocks_swig2.argmax_ss_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(argmax_ss_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.argmax_ss_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(argmax_ss_sptr self) -> int"""
        return _blocks_swig2.argmax_ss_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(argmax_ss_sptr self) -> int"""
        return _blocks_swig2.argmax_ss_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(argmax_ss_sptr self, int priority) -> int"""
        return _blocks_swig2.argmax_ss_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(argmax_ss_sptr self) -> std::string"""
        return _blocks_swig2.argmax_ss_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(argmax_ss_sptr self) -> std::string"""
        return _blocks_swig2.argmax_ss_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(argmax_ss_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.argmax_ss_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(argmax_ss_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.argmax_ss_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(argmax_ss_sptr self) -> long"""
        return _blocks_swig2.argmax_ss_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(argmax_ss_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.argmax_ss_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(argmax_ss_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.argmax_ss_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(argmax_ss_sptr self) -> std::string"""
        return _blocks_swig2.argmax_ss_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(argmax_ss_sptr self, std::string name)"""
        return _blocks_swig2.argmax_ss_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(argmax_ss_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.argmax_ss_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(argmax_ss_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_ss_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(argmax_ss_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_ss_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(argmax_ss_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.argmax_ss_sptr_message_subscribers(self, which_port)

# Register argmax_ss_sptr in _blocks_swig2:
_blocks_swig2.argmax_ss_sptr_swigregister(argmax_ss_sptr)


argmax_ss_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
argmax_ss = argmax_ss.make

class bin_statistics_f_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::bin_statistics_f > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(bin_statistics_f_sptr self) -> bin_statistics_f_sptr
        __init__(bin_statistics_f_sptr self, bin_statistics_f p) -> bin_statistics_f_sptr
        """
        _blocks_swig2.bin_statistics_f_sptr_swiginit(self, _blocks_swig2.new_bin_statistics_f_sptr(*args))

    def __deref__(self) -> "gr::blocks::bin_statistics_f *":
        r"""__deref__(bin_statistics_f_sptr self) -> bin_statistics_f"""
        return _blocks_swig2.bin_statistics_f_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_bin_statistics_f_sptr

    def make(self, vlen: "unsigned int", msgq: "msg_queue_sptr", tune: "gr::feval_dd *", tune_delay: "size_t", dwell_delay: "size_t") -> "gr::blocks::bin_statistics_f::sptr":
        r"""make(bin_statistics_f_sptr self, unsigned int vlen, msg_queue_sptr msgq, gr::feval_dd * tune, size_t tune_delay, size_t dwell_delay) -> bin_statistics_f_sptr"""
        return _blocks_swig2.bin_statistics_f_sptr_make(self, vlen, msgq, tune, tune_delay, dwell_delay)

    def history(self) -> "unsigned int":
        r"""history(bin_statistics_f_sptr self) -> unsigned int"""
        return _blocks_swig2.bin_statistics_f_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(bin_statistics_f_sptr self, int which, int delay)
        declare_sample_delay(bin_statistics_f_sptr self, unsigned int delay)
        """
        return _blocks_swig2.bin_statistics_f_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(bin_statistics_f_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.bin_statistics_f_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(bin_statistics_f_sptr self, int multiple)"""
        return _blocks_swig2.bin_statistics_f_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(bin_statistics_f_sptr self) -> int"""
        return _blocks_swig2.bin_statistics_f_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(bin_statistics_f_sptr self) -> double"""
        return _blocks_swig2.bin_statistics_f_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(bin_statistics_f_sptr self) -> uint64_t"""
        return _blocks_swig2.bin_statistics_f_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(bin_statistics_f_sptr self) -> uint64_t"""
        return _blocks_swig2.bin_statistics_f_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(bin_statistics_f_sptr self) -> bool"""
        return _blocks_swig2.bin_statistics_f_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(bin_statistics_f_sptr self) -> bool"""
        return _blocks_swig2.bin_statistics_f_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(bin_statistics_f_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.bin_statistics_f_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(bin_statistics_f_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.bin_statistics_f_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(bin_statistics_f_sptr self, std::string level)"""
        return _blocks_swig2.bin_statistics_f_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(bin_statistics_f_sptr self) -> std::string"""
        return _blocks_swig2.bin_statistics_f_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(bin_statistics_f_sptr self) -> int"""
        return _blocks_swig2.bin_statistics_f_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(bin_statistics_f_sptr self, int m)"""
        return _blocks_swig2.bin_statistics_f_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(bin_statistics_f_sptr self)"""
        return _blocks_swig2.bin_statistics_f_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(bin_statistics_f_sptr self) -> bool"""
        return _blocks_swig2.bin_statistics_f_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(bin_statistics_f_sptr self, int m)"""
        return _blocks_swig2.bin_statistics_f_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(bin_statistics_f_sptr self) -> int"""
        return _blocks_swig2.bin_statistics_f_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(bin_statistics_f_sptr self, int i) -> long"""
        return _blocks_swig2.bin_statistics_f_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(bin_statistics_f_sptr self, long max_output_buffer)
        set_max_output_buffer(bin_statistics_f_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.bin_statistics_f_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(bin_statistics_f_sptr self, int i) -> long"""
        return _blocks_swig2.bin_statistics_f_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(bin_statistics_f_sptr self, long min_output_buffer)
        set_min_output_buffer(bin_statistics_f_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.bin_statistics_f_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(bin_statistics_f_sptr self, int which) -> float
        pc_input_buffers_full(bin_statistics_f_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.bin_statistics_f_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(bin_statistics_f_sptr self, int which) -> float
        pc_input_buffers_full_avg(bin_statistics_f_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.bin_statistics_f_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(bin_statistics_f_sptr self, int which) -> float
        pc_input_buffers_full_var(bin_statistics_f_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.bin_statistics_f_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(bin_statistics_f_sptr self, int which) -> float
        pc_output_buffers_full(bin_statistics_f_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.bin_statistics_f_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(bin_statistics_f_sptr self, int which) -> float
        pc_output_buffers_full_avg(bin_statistics_f_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.bin_statistics_f_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(bin_statistics_f_sptr self, int which) -> float
        pc_output_buffers_full_var(bin_statistics_f_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.bin_statistics_f_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(bin_statistics_f_sptr self) -> float"""
        return _blocks_swig2.bin_statistics_f_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(bin_statistics_f_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.bin_statistics_f_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(bin_statistics_f_sptr self)"""
        return _blocks_swig2.bin_statistics_f_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(bin_statistics_f_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.bin_statistics_f_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(bin_statistics_f_sptr self) -> int"""
        return _blocks_swig2.bin_statistics_f_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(bin_statistics_f_sptr self) -> int"""
        return _blocks_swig2.bin_statistics_f_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(bin_statistics_f_sptr self, int priority) -> int"""
        return _blocks_swig2.bin_statistics_f_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(bin_statistics_f_sptr self) -> std::string"""
        return _blocks_swig2.bin_statistics_f_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(bin_statistics_f_sptr self) -> std::string"""
        return _blocks_swig2.bin_statistics_f_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(bin_statistics_f_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.bin_statistics_f_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(bin_statistics_f_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.bin_statistics_f_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(bin_statistics_f_sptr self) -> long"""
        return _blocks_swig2.bin_statistics_f_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(bin_statistics_f_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.bin_statistics_f_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(bin_statistics_f_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.bin_statistics_f_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(bin_statistics_f_sptr self) -> std::string"""
        return _blocks_swig2.bin_statistics_f_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(bin_statistics_f_sptr self, std::string name)"""
        return _blocks_swig2.bin_statistics_f_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(bin_statistics_f_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.bin_statistics_f_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(bin_statistics_f_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.bin_statistics_f_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(bin_statistics_f_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.bin_statistics_f_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(bin_statistics_f_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.bin_statistics_f_sptr_message_subscribers(self, which_port)

# Register bin_statistics_f_sptr in _blocks_swig2:
_blocks_swig2.bin_statistics_f_sptr_swigregister(bin_statistics_f_sptr)


bin_statistics_f_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
bin_statistics_f = bin_statistics_f.make;

class burst_tagger_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::burst_tagger > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(burst_tagger_sptr self) -> burst_tagger_sptr
        __init__(burst_tagger_sptr self, burst_tagger p) -> burst_tagger_sptr
        """
        _blocks_swig2.burst_tagger_sptr_swiginit(self, _blocks_swig2.new_burst_tagger_sptr(*args))

    def __deref__(self) -> "gr::blocks::burst_tagger *":
        r"""__deref__(burst_tagger_sptr self) -> burst_tagger"""
        return _blocks_swig2.burst_tagger_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_burst_tagger_sptr

    def make(self, itemsize: "size_t") -> "gr::blocks::burst_tagger::sptr":
        r"""make(burst_tagger_sptr self, size_t itemsize) -> burst_tagger_sptr"""
        return _blocks_swig2.burst_tagger_sptr_make(self, itemsize)

    def set_true_tag(self, key: "std::string const &", value: "bool") -> "void":
        r"""set_true_tag(burst_tagger_sptr self, std::string const & key, bool value)"""
        return _blocks_swig2.burst_tagger_sptr_set_true_tag(self, key, value)

    def set_false_tag(self, key: "std::string const &", value: "bool") -> "void":
        r"""set_false_tag(burst_tagger_sptr self, std::string const & key, bool value)"""
        return _blocks_swig2.burst_tagger_sptr_set_false_tag(self, key, value)

    def history(self) -> "unsigned int":
        r"""history(burst_tagger_sptr self) -> unsigned int"""
        return _blocks_swig2.burst_tagger_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(burst_tagger_sptr self, int which, int delay)
        declare_sample_delay(burst_tagger_sptr self, unsigned int delay)
        """
        return _blocks_swig2.burst_tagger_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(burst_tagger_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.burst_tagger_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(burst_tagger_sptr self, int multiple)"""
        return _blocks_swig2.burst_tagger_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(burst_tagger_sptr self) -> int"""
        return _blocks_swig2.burst_tagger_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(burst_tagger_sptr self) -> double"""
        return _blocks_swig2.burst_tagger_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(burst_tagger_sptr self) -> uint64_t"""
        return _blocks_swig2.burst_tagger_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(burst_tagger_sptr self) -> uint64_t"""
        return _blocks_swig2.burst_tagger_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(burst_tagger_sptr self) -> bool"""
        return _blocks_swig2.burst_tagger_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(burst_tagger_sptr self) -> bool"""
        return _blocks_swig2.burst_tagger_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(burst_tagger_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.burst_tagger_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(burst_tagger_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.burst_tagger_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(burst_tagger_sptr self, std::string level)"""
        return _blocks_swig2.burst_tagger_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(burst_tagger_sptr self) -> std::string"""
        return _blocks_swig2.burst_tagger_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(burst_tagger_sptr self) -> int"""
        return _blocks_swig2.burst_tagger_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(burst_tagger_sptr self, int m)"""
        return _blocks_swig2.burst_tagger_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(burst_tagger_sptr self)"""
        return _blocks_swig2.burst_tagger_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(burst_tagger_sptr self) -> bool"""
        return _blocks_swig2.burst_tagger_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(burst_tagger_sptr self, int m)"""
        return _blocks_swig2.burst_tagger_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(burst_tagger_sptr self) -> int"""
        return _blocks_swig2.burst_tagger_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(burst_tagger_sptr self, int i) -> long"""
        return _blocks_swig2.burst_tagger_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(burst_tagger_sptr self, long max_output_buffer)
        set_max_output_buffer(burst_tagger_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.burst_tagger_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(burst_tagger_sptr self, int i) -> long"""
        return _blocks_swig2.burst_tagger_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(burst_tagger_sptr self, long min_output_buffer)
        set_min_output_buffer(burst_tagger_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.burst_tagger_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(burst_tagger_sptr self, int which) -> float
        pc_input_buffers_full(burst_tagger_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.burst_tagger_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(burst_tagger_sptr self, int which) -> float
        pc_input_buffers_full_avg(burst_tagger_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.burst_tagger_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(burst_tagger_sptr self, int which) -> float
        pc_input_buffers_full_var(burst_tagger_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.burst_tagger_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(burst_tagger_sptr self, int which) -> float
        pc_output_buffers_full(burst_tagger_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.burst_tagger_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(burst_tagger_sptr self, int which) -> float
        pc_output_buffers_full_avg(burst_tagger_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.burst_tagger_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(burst_tagger_sptr self, int which) -> float
        pc_output_buffers_full_var(burst_tagger_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.burst_tagger_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(burst_tagger_sptr self) -> float"""
        return _blocks_swig2.burst_tagger_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(burst_tagger_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.burst_tagger_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(burst_tagger_sptr self)"""
        return _blocks_swig2.burst_tagger_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(burst_tagger_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.burst_tagger_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(burst_tagger_sptr self) -> int"""
        return _blocks_swig2.burst_tagger_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(burst_tagger_sptr self) -> int"""
        return _blocks_swig2.burst_tagger_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(burst_tagger_sptr self, int priority) -> int"""
        return _blocks_swig2.burst_tagger_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(burst_tagger_sptr self) -> std::string"""
        return _blocks_swig2.burst_tagger_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(burst_tagger_sptr self) -> std::string"""
        return _blocks_swig2.burst_tagger_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(burst_tagger_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.burst_tagger_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(burst_tagger_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.burst_tagger_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(burst_tagger_sptr self) -> long"""
        return _blocks_swig2.burst_tagger_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(burst_tagger_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.burst_tagger_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(burst_tagger_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.burst_tagger_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(burst_tagger_sptr self) -> std::string"""
        return _blocks_swig2.burst_tagger_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(burst_tagger_sptr self, std::string name)"""
        return _blocks_swig2.burst_tagger_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(burst_tagger_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.burst_tagger_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(burst_tagger_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.burst_tagger_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(burst_tagger_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.burst_tagger_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(burst_tagger_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.burst_tagger_sptr_message_subscribers(self, which_port)

# Register burst_tagger_sptr in _blocks_swig2:
_blocks_swig2.burst_tagger_sptr_swigregister(burst_tagger_sptr)


burst_tagger_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
burst_tagger = burst_tagger.make;

class char_to_float_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::char_to_float > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(char_to_float_sptr self) -> char_to_float_sptr
        __init__(char_to_float_sptr self, char_to_float p) -> char_to_float_sptr
        """
        _blocks_swig2.char_to_float_sptr_swiginit(self, _blocks_swig2.new_char_to_float_sptr(*args))

    def __deref__(self) -> "gr::blocks::char_to_float *":
        r"""__deref__(char_to_float_sptr self) -> char_to_float"""
        return _blocks_swig2.char_to_float_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_char_to_float_sptr

    def make(self, vlen: "size_t"=1, scale: "float"=1.0) -> "gr::blocks::char_to_float::sptr":
        r"""make(char_to_float_sptr self, size_t vlen=1, float scale=1.0) -> char_to_float_sptr"""
        return _blocks_swig2.char_to_float_sptr_make(self, vlen, scale)

    def scale(self) -> "float":
        r"""scale(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_scale(self)

    def set_scale(self, scale: "float") -> "void":
        r"""set_scale(char_to_float_sptr self, float scale)"""
        return _blocks_swig2.char_to_float_sptr_set_scale(self, scale)

    def history(self) -> "unsigned int":
        r"""history(char_to_float_sptr self) -> unsigned int"""
        return _blocks_swig2.char_to_float_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(char_to_float_sptr self, int which, int delay)
        declare_sample_delay(char_to_float_sptr self, unsigned int delay)
        """
        return _blocks_swig2.char_to_float_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(char_to_float_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.char_to_float_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(char_to_float_sptr self, int multiple)"""
        return _blocks_swig2.char_to_float_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(char_to_float_sptr self) -> int"""
        return _blocks_swig2.char_to_float_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(char_to_float_sptr self) -> double"""
        return _blocks_swig2.char_to_float_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(char_to_float_sptr self) -> uint64_t"""
        return _blocks_swig2.char_to_float_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(char_to_float_sptr self) -> uint64_t"""
        return _blocks_swig2.char_to_float_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(char_to_float_sptr self) -> bool"""
        return _blocks_swig2.char_to_float_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(char_to_float_sptr self) -> bool"""
        return _blocks_swig2.char_to_float_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(char_to_float_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.char_to_float_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(char_to_float_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.char_to_float_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(char_to_float_sptr self, std::string level)"""
        return _blocks_swig2.char_to_float_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(char_to_float_sptr self) -> std::string"""
        return _blocks_swig2.char_to_float_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(char_to_float_sptr self) -> int"""
        return _blocks_swig2.char_to_float_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(char_to_float_sptr self, int m)"""
        return _blocks_swig2.char_to_float_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(char_to_float_sptr self)"""
        return _blocks_swig2.char_to_float_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(char_to_float_sptr self) -> bool"""
        return _blocks_swig2.char_to_float_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(char_to_float_sptr self, int m)"""
        return _blocks_swig2.char_to_float_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(char_to_float_sptr self) -> int"""
        return _blocks_swig2.char_to_float_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(char_to_float_sptr self, int i) -> long"""
        return _blocks_swig2.char_to_float_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(char_to_float_sptr self, long max_output_buffer)
        set_max_output_buffer(char_to_float_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.char_to_float_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(char_to_float_sptr self, int i) -> long"""
        return _blocks_swig2.char_to_float_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(char_to_float_sptr self, long min_output_buffer)
        set_min_output_buffer(char_to_float_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.char_to_float_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(char_to_float_sptr self, int which) -> float
        pc_input_buffers_full(char_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_float_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(char_to_float_sptr self, int which) -> float
        pc_input_buffers_full_avg(char_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_float_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(char_to_float_sptr self, int which) -> float
        pc_input_buffers_full_var(char_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_float_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(char_to_float_sptr self, int which) -> float
        pc_output_buffers_full(char_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_float_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(char_to_float_sptr self, int which) -> float
        pc_output_buffers_full_avg(char_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_float_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(char_to_float_sptr self, int which) -> float
        pc_output_buffers_full_var(char_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_float_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(char_to_float_sptr self) -> float"""
        return _blocks_swig2.char_to_float_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(char_to_float_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.char_to_float_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(char_to_float_sptr self)"""
        return _blocks_swig2.char_to_float_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(char_to_float_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.char_to_float_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(char_to_float_sptr self) -> int"""
        return _blocks_swig2.char_to_float_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(char_to_float_sptr self) -> int"""
        return _blocks_swig2.char_to_float_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(char_to_float_sptr self, int priority) -> int"""
        return _blocks_swig2.char_to_float_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(char_to_float_sptr self) -> std::string"""
        return _blocks_swig2.char_to_float_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(char_to_float_sptr self) -> std::string"""
        return _blocks_swig2.char_to_float_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(char_to_float_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.char_to_float_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(char_to_float_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.char_to_float_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(char_to_float_sptr self) -> long"""
        return _blocks_swig2.char_to_float_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(char_to_float_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.char_to_float_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(char_to_float_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.char_to_float_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(char_to_float_sptr self) -> std::string"""
        return _blocks_swig2.char_to_float_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(char_to_float_sptr self, std::string name)"""
        return _blocks_swig2.char_to_float_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(char_to_float_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.char_to_float_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(char_to_float_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.char_to_float_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(char_to_float_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.char_to_float_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(char_to_float_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.char_to_float_sptr_message_subscribers(self, which_port)

# Register char_to_float_sptr in _blocks_swig2:
_blocks_swig2.char_to_float_sptr_swigregister(char_to_float_sptr)


char_to_float_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
char_to_float = char_to_float.make;

class char_to_short_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::char_to_short > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(char_to_short_sptr self) -> char_to_short_sptr
        __init__(char_to_short_sptr self, char_to_short p) -> char_to_short_sptr
        """
        _blocks_swig2.char_to_short_sptr_swiginit(self, _blocks_swig2.new_char_to_short_sptr(*args))

    def __deref__(self) -> "gr::blocks::char_to_short *":
        r"""__deref__(char_to_short_sptr self) -> char_to_short"""
        return _blocks_swig2.char_to_short_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_char_to_short_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::char_to_short::sptr":
        r"""make(char_to_short_sptr self, size_t vlen=1) -> char_to_short_sptr"""
        return _blocks_swig2.char_to_short_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(char_to_short_sptr self) -> unsigned int"""
        return _blocks_swig2.char_to_short_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(char_to_short_sptr self, int which, int delay)
        declare_sample_delay(char_to_short_sptr self, unsigned int delay)
        """
        return _blocks_swig2.char_to_short_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(char_to_short_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.char_to_short_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(char_to_short_sptr self, int multiple)"""
        return _blocks_swig2.char_to_short_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(char_to_short_sptr self) -> int"""
        return _blocks_swig2.char_to_short_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(char_to_short_sptr self) -> double"""
        return _blocks_swig2.char_to_short_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(char_to_short_sptr self) -> uint64_t"""
        return _blocks_swig2.char_to_short_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(char_to_short_sptr self) -> uint64_t"""
        return _blocks_swig2.char_to_short_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(char_to_short_sptr self) -> bool"""
        return _blocks_swig2.char_to_short_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(char_to_short_sptr self) -> bool"""
        return _blocks_swig2.char_to_short_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(char_to_short_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.char_to_short_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(char_to_short_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.char_to_short_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(char_to_short_sptr self, std::string level)"""
        return _blocks_swig2.char_to_short_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(char_to_short_sptr self) -> std::string"""
        return _blocks_swig2.char_to_short_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(char_to_short_sptr self) -> int"""
        return _blocks_swig2.char_to_short_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(char_to_short_sptr self, int m)"""
        return _blocks_swig2.char_to_short_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(char_to_short_sptr self)"""
        return _blocks_swig2.char_to_short_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(char_to_short_sptr self) -> bool"""
        return _blocks_swig2.char_to_short_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(char_to_short_sptr self, int m)"""
        return _blocks_swig2.char_to_short_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(char_to_short_sptr self) -> int"""
        return _blocks_swig2.char_to_short_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(char_to_short_sptr self, int i) -> long"""
        return _blocks_swig2.char_to_short_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(char_to_short_sptr self, long max_output_buffer)
        set_max_output_buffer(char_to_short_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.char_to_short_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(char_to_short_sptr self, int i) -> long"""
        return _blocks_swig2.char_to_short_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(char_to_short_sptr self, long min_output_buffer)
        set_min_output_buffer(char_to_short_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.char_to_short_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(char_to_short_sptr self, int which) -> float
        pc_input_buffers_full(char_to_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_short_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(char_to_short_sptr self, int which) -> float
        pc_input_buffers_full_avg(char_to_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_short_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(char_to_short_sptr self, int which) -> float
        pc_input_buffers_full_var(char_to_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_short_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(char_to_short_sptr self, int which) -> float
        pc_output_buffers_full(char_to_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_short_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(char_to_short_sptr self, int which) -> float
        pc_output_buffers_full_avg(char_to_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_short_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(char_to_short_sptr self, int which) -> float
        pc_output_buffers_full_var(char_to_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.char_to_short_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(char_to_short_sptr self) -> float"""
        return _blocks_swig2.char_to_short_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(char_to_short_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.char_to_short_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(char_to_short_sptr self)"""
        return _blocks_swig2.char_to_short_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(char_to_short_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.char_to_short_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(char_to_short_sptr self) -> int"""
        return _blocks_swig2.char_to_short_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(char_to_short_sptr self) -> int"""
        return _blocks_swig2.char_to_short_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(char_to_short_sptr self, int priority) -> int"""
        return _blocks_swig2.char_to_short_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(char_to_short_sptr self) -> std::string"""
        return _blocks_swig2.char_to_short_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(char_to_short_sptr self) -> std::string"""
        return _blocks_swig2.char_to_short_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(char_to_short_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.char_to_short_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(char_to_short_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.char_to_short_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(char_to_short_sptr self) -> long"""
        return _blocks_swig2.char_to_short_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(char_to_short_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.char_to_short_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(char_to_short_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.char_to_short_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(char_to_short_sptr self) -> std::string"""
        return _blocks_swig2.char_to_short_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(char_to_short_sptr self, std::string name)"""
        return _blocks_swig2.char_to_short_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(char_to_short_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.char_to_short_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(char_to_short_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.char_to_short_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(char_to_short_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.char_to_short_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(char_to_short_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.char_to_short_sptr_message_subscribers(self, which_port)

# Register char_to_short_sptr in _blocks_swig2:
_blocks_swig2.char_to_short_sptr_swigregister(char_to_short_sptr)


char_to_short_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
char_to_short = char_to_short.make;

class check_lfsr_32k_s_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::check_lfsr_32k_s > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(check_lfsr_32k_s_sptr self) -> check_lfsr_32k_s_sptr
        __init__(check_lfsr_32k_s_sptr self, check_lfsr_32k_s p) -> check_lfsr_32k_s_sptr
        """
        _blocks_swig2.check_lfsr_32k_s_sptr_swiginit(self, _blocks_swig2.new_check_lfsr_32k_s_sptr(*args))

    def __deref__(self) -> "gr::blocks::check_lfsr_32k_s *":
        r"""__deref__(check_lfsr_32k_s_sptr self) -> check_lfsr_32k_s"""
        return _blocks_swig2.check_lfsr_32k_s_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_check_lfsr_32k_s_sptr

    def make(self) -> "gr::blocks::check_lfsr_32k_s::sptr":
        r"""make(check_lfsr_32k_s_sptr self) -> check_lfsr_32k_s_sptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_make(self)

    def ntotal(self) -> "long":
        r"""ntotal(check_lfsr_32k_s_sptr self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_ntotal(self)

    def nright(self) -> "long":
        r"""nright(check_lfsr_32k_s_sptr self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_nright(self)

    def runlength(self) -> "long":
        r"""runlength(check_lfsr_32k_s_sptr self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_runlength(self)

    def history(self) -> "unsigned int":
        r"""history(check_lfsr_32k_s_sptr self) -> unsigned int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(check_lfsr_32k_s_sptr self, int which, int delay)
        declare_sample_delay(check_lfsr_32k_s_sptr self, unsigned int delay)
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(check_lfsr_32k_s_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(check_lfsr_32k_s_sptr self, int multiple)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(check_lfsr_32k_s_sptr self) -> int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(check_lfsr_32k_s_sptr self) -> double"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(check_lfsr_32k_s_sptr self) -> uint64_t"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(check_lfsr_32k_s_sptr self) -> uint64_t"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(check_lfsr_32k_s_sptr self) -> bool"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(check_lfsr_32k_s_sptr self) -> bool"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(check_lfsr_32k_s_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(check_lfsr_32k_s_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(check_lfsr_32k_s_sptr self, std::string level)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(check_lfsr_32k_s_sptr self) -> std::string"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(check_lfsr_32k_s_sptr self) -> int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(check_lfsr_32k_s_sptr self, int m)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(check_lfsr_32k_s_sptr self)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(check_lfsr_32k_s_sptr self) -> bool"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(check_lfsr_32k_s_sptr self, int m)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(check_lfsr_32k_s_sptr self) -> int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(check_lfsr_32k_s_sptr self, int i) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(check_lfsr_32k_s_sptr self, long max_output_buffer)
        set_max_output_buffer(check_lfsr_32k_s_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(check_lfsr_32k_s_sptr self, int i) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(check_lfsr_32k_s_sptr self, long min_output_buffer)
        set_min_output_buffer(check_lfsr_32k_s_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(check_lfsr_32k_s_sptr self, int which) -> float
        pc_input_buffers_full(check_lfsr_32k_s_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(check_lfsr_32k_s_sptr self, int which) -> float
        pc_input_buffers_full_avg(check_lfsr_32k_s_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(check_lfsr_32k_s_sptr self, int which) -> float
        pc_input_buffers_full_var(check_lfsr_32k_s_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(check_lfsr_32k_s_sptr self, int which) -> float
        pc_output_buffers_full(check_lfsr_32k_s_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(check_lfsr_32k_s_sptr self, int which) -> float
        pc_output_buffers_full_avg(check_lfsr_32k_s_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(check_lfsr_32k_s_sptr self, int which) -> float
        pc_output_buffers_full_var(check_lfsr_32k_s_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(check_lfsr_32k_s_sptr self) -> float"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(check_lfsr_32k_s_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(check_lfsr_32k_s_sptr self)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(check_lfsr_32k_s_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(check_lfsr_32k_s_sptr self) -> int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(check_lfsr_32k_s_sptr self) -> int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(check_lfsr_32k_s_sptr self, int priority) -> int"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(check_lfsr_32k_s_sptr self) -> std::string"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(check_lfsr_32k_s_sptr self) -> std::string"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(check_lfsr_32k_s_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(check_lfsr_32k_s_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(check_lfsr_32k_s_sptr self) -> long"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(check_lfsr_32k_s_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(check_lfsr_32k_s_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(check_lfsr_32k_s_sptr self) -> std::string"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(check_lfsr_32k_s_sptr self, std::string name)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(check_lfsr_32k_s_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.check_lfsr_32k_s_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(check_lfsr_32k_s_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(check_lfsr_32k_s_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(check_lfsr_32k_s_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.check_lfsr_32k_s_sptr_message_subscribers(self, which_port)

# Register check_lfsr_32k_s_sptr in _blocks_swig2:
_blocks_swig2.check_lfsr_32k_s_sptr_swigregister(check_lfsr_32k_s_sptr)


check_lfsr_32k_s_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
check_lfsr_32k_s = check_lfsr_32k_s.make;

class complex_to_interleaved_char_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_interleaved_char > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_interleaved_char_sptr self) -> complex_to_interleaved_char_sptr
        __init__(complex_to_interleaved_char_sptr self, complex_to_interleaved_char p) -> complex_to_interleaved_char_sptr
        """
        _blocks_swig2.complex_to_interleaved_char_sptr_swiginit(self, _blocks_swig2.new_complex_to_interleaved_char_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_interleaved_char *":
        r"""__deref__(complex_to_interleaved_char_sptr self) -> complex_to_interleaved_char"""
        return _blocks_swig2.complex_to_interleaved_char_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_interleaved_char_sptr

    def make(self, vector: "bool"=False) -> "gr::blocks::complex_to_interleaved_char::sptr":
        r"""make(complex_to_interleaved_char_sptr self, bool vector=False) -> complex_to_interleaved_char_sptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_make(self, vector)

    def history(self) -> "unsigned int":
        r"""history(complex_to_interleaved_char_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_interleaved_char_sptr self, int which, int delay)
        declare_sample_delay(complex_to_interleaved_char_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_interleaved_char_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_interleaved_char_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_interleaved_char_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_interleaved_char_sptr self) -> double"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_interleaved_char_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_interleaved_char_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_interleaved_char_sptr self) -> bool"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_interleaved_char_sptr self) -> bool"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_interleaved_char_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_interleaved_char_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_interleaved_char_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_interleaved_char_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_interleaved_char_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_interleaved_char_sptr self, int m)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_interleaved_char_sptr self)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_interleaved_char_sptr self) -> bool"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_interleaved_char_sptr self, int m)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_interleaved_char_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_interleaved_char_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_interleaved_char_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_interleaved_char_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_interleaved_char_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_interleaved_char_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_interleaved_char_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_interleaved_char_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_interleaved_char_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_interleaved_char_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_interleaved_char_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_interleaved_char_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_interleaved_char_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_interleaved_char_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_interleaved_char_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_interleaved_char_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_interleaved_char_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_interleaved_char_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_interleaved_char_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_interleaved_char_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_interleaved_char_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_interleaved_char_sptr self)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_interleaved_char_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_interleaved_char_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_interleaved_char_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_interleaved_char_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_interleaved_char_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_interleaved_char_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_interleaved_char_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_interleaved_char_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_interleaved_char_sptr self) -> long"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_interleaved_char_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_interleaved_char_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_interleaved_char_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_interleaved_char_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_interleaved_char_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_interleaved_char_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_interleaved_char_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_interleaved_char_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_interleaved_char_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_interleaved_char_sptr_message_subscribers(self, which_port)

# Register complex_to_interleaved_char_sptr in _blocks_swig2:
_blocks_swig2.complex_to_interleaved_char_sptr_swigregister(complex_to_interleaved_char_sptr)


complex_to_interleaved_char_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_interleaved_char = complex_to_interleaved_char.make;

class complex_to_interleaved_short_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_interleaved_short > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_interleaved_short_sptr self) -> complex_to_interleaved_short_sptr
        __init__(complex_to_interleaved_short_sptr self, complex_to_interleaved_short p) -> complex_to_interleaved_short_sptr
        """
        _blocks_swig2.complex_to_interleaved_short_sptr_swiginit(self, _blocks_swig2.new_complex_to_interleaved_short_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_interleaved_short *":
        r"""__deref__(complex_to_interleaved_short_sptr self) -> complex_to_interleaved_short"""
        return _blocks_swig2.complex_to_interleaved_short_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_interleaved_short_sptr

    def make(self, vector: "bool"=False) -> "gr::blocks::complex_to_interleaved_short::sptr":
        r"""make(complex_to_interleaved_short_sptr self, bool vector=False) -> complex_to_interleaved_short_sptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_make(self, vector)

    def history(self) -> "unsigned int":
        r"""history(complex_to_interleaved_short_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_interleaved_short_sptr self, int which, int delay)
        declare_sample_delay(complex_to_interleaved_short_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_interleaved_short_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_interleaved_short_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_interleaved_short_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_interleaved_short_sptr self) -> double"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_interleaved_short_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_interleaved_short_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_interleaved_short_sptr self) -> bool"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_interleaved_short_sptr self) -> bool"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_interleaved_short_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_interleaved_short_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_interleaved_short_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_interleaved_short_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_interleaved_short_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_interleaved_short_sptr self, int m)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_interleaved_short_sptr self)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_interleaved_short_sptr self) -> bool"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_interleaved_short_sptr self, int m)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_interleaved_short_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_interleaved_short_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_interleaved_short_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_interleaved_short_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_interleaved_short_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_interleaved_short_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_interleaved_short_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_interleaved_short_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_interleaved_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_interleaved_short_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_interleaved_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_interleaved_short_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_interleaved_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_interleaved_short_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_interleaved_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_interleaved_short_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_interleaved_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_interleaved_short_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_interleaved_short_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_interleaved_short_sptr self) -> float"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_interleaved_short_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_interleaved_short_sptr self)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_interleaved_short_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_interleaved_short_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_interleaved_short_sptr self) -> int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_interleaved_short_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_interleaved_short_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_interleaved_short_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_interleaved_short_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_interleaved_short_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_interleaved_short_sptr self) -> long"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_interleaved_short_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_interleaved_short_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_interleaved_short_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_interleaved_short_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_interleaved_short_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_interleaved_short_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_interleaved_short_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_interleaved_short_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_interleaved_short_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_interleaved_short_sptr_message_subscribers(self, which_port)

# Register complex_to_interleaved_short_sptr in _blocks_swig2:
_blocks_swig2.complex_to_interleaved_short_sptr_swigregister(complex_to_interleaved_short_sptr)


complex_to_interleaved_short_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_interleaved_short = complex_to_interleaved_short.make;

class complex_to_float_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_float > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_float_sptr self) -> complex_to_float_sptr
        __init__(complex_to_float_sptr self, complex_to_float p) -> complex_to_float_sptr
        """
        _blocks_swig2.complex_to_float_sptr_swiginit(self, _blocks_swig2.new_complex_to_float_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_float *":
        r"""__deref__(complex_to_float_sptr self) -> complex_to_float"""
        return _blocks_swig2.complex_to_float_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_float_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_float::sptr":
        r"""make(complex_to_float_sptr self, size_t vlen=1) -> complex_to_float_sptr"""
        return _blocks_swig2.complex_to_float_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_float_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_float_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_float_sptr self, int which, int delay)
        declare_sample_delay(complex_to_float_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_float_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_float_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_float_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_float_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_float_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_float_sptr self) -> int"""
        return _blocks_swig2.complex_to_float_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_float_sptr self) -> double"""
        return _blocks_swig2.complex_to_float_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_float_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_float_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_float_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_float_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_float_sptr self) -> bool"""
        return _blocks_swig2.complex_to_float_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_float_sptr self) -> bool"""
        return _blocks_swig2.complex_to_float_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_float_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_float_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_float_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_float_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_float_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_float_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_float_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_float_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_float_sptr self) -> int"""
        return _blocks_swig2.complex_to_float_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_float_sptr self, int m)"""
        return _blocks_swig2.complex_to_float_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_float_sptr self)"""
        return _blocks_swig2.complex_to_float_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_float_sptr self) -> bool"""
        return _blocks_swig2.complex_to_float_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_float_sptr self, int m)"""
        return _blocks_swig2.complex_to_float_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_float_sptr self) -> int"""
        return _blocks_swig2.complex_to_float_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_float_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_float_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_float_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_float_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_float_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_float_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_float_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_float_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_float_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_float_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_float_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_float_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_float_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_float_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_float_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_float_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_float_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_float_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_float_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_float_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_float_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_float_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_float_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_float_sptr self) -> float"""
        return _blocks_swig2.complex_to_float_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_float_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_float_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_float_sptr self)"""
        return _blocks_swig2.complex_to_float_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_float_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_float_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_float_sptr self) -> int"""
        return _blocks_swig2.complex_to_float_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_float_sptr self) -> int"""
        return _blocks_swig2.complex_to_float_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_float_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_float_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_float_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_float_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_float_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_float_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_float_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_float_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_float_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_float_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_float_sptr self) -> long"""
        return _blocks_swig2.complex_to_float_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_float_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_float_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_float_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_float_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_float_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_float_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_float_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_float_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_float_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_float_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_float_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_float_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_float_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_float_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_float_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_float_sptr_message_subscribers(self, which_port)

# Register complex_to_float_sptr in _blocks_swig2:
_blocks_swig2.complex_to_float_sptr_swigregister(complex_to_float_sptr)


complex_to_float_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_float = complex_to_float.make;

class complex_to_magphase_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_magphase > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_magphase_sptr self) -> complex_to_magphase_sptr
        __init__(complex_to_magphase_sptr self, complex_to_magphase p) -> complex_to_magphase_sptr
        """
        _blocks_swig2.complex_to_magphase_sptr_swiginit(self, _blocks_swig2.new_complex_to_magphase_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_magphase *":
        r"""__deref__(complex_to_magphase_sptr self) -> complex_to_magphase"""
        return _blocks_swig2.complex_to_magphase_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_magphase_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_magphase::sptr":
        r"""make(complex_to_magphase_sptr self, size_t vlen=1) -> complex_to_magphase_sptr"""
        return _blocks_swig2.complex_to_magphase_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_magphase_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_magphase_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_magphase_sptr self, int which, int delay)
        declare_sample_delay(complex_to_magphase_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_magphase_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_magphase_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_magphase_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_magphase_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_magphase_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_magphase_sptr self) -> int"""
        return _blocks_swig2.complex_to_magphase_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_magphase_sptr self) -> double"""
        return _blocks_swig2.complex_to_magphase_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_magphase_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_magphase_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_magphase_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_magphase_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_magphase_sptr self) -> bool"""
        return _blocks_swig2.complex_to_magphase_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_magphase_sptr self) -> bool"""
        return _blocks_swig2.complex_to_magphase_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_magphase_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_magphase_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_magphase_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_magphase_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_magphase_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_magphase_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_magphase_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_magphase_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_magphase_sptr self) -> int"""
        return _blocks_swig2.complex_to_magphase_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_magphase_sptr self, int m)"""
        return _blocks_swig2.complex_to_magphase_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_magphase_sptr self)"""
        return _blocks_swig2.complex_to_magphase_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_magphase_sptr self) -> bool"""
        return _blocks_swig2.complex_to_magphase_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_magphase_sptr self, int m)"""
        return _blocks_swig2.complex_to_magphase_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_magphase_sptr self) -> int"""
        return _blocks_swig2.complex_to_magphase_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_magphase_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_magphase_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_magphase_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_magphase_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_magphase_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_magphase_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_magphase_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_magphase_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_magphase_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_magphase_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_magphase_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_magphase_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_magphase_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_magphase_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_magphase_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_magphase_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_magphase_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_magphase_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_magphase_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_magphase_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_magphase_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_magphase_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_magphase_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_magphase_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_magphase_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_magphase_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_magphase_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_magphase_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_magphase_sptr self) -> float"""
        return _blocks_swig2.complex_to_magphase_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_magphase_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_magphase_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_magphase_sptr self)"""
        return _blocks_swig2.complex_to_magphase_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_magphase_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_magphase_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_magphase_sptr self) -> int"""
        return _blocks_swig2.complex_to_magphase_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_magphase_sptr self) -> int"""
        return _blocks_swig2.complex_to_magphase_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_magphase_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_magphase_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_magphase_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_magphase_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_magphase_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_magphase_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_magphase_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_magphase_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_magphase_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_magphase_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_magphase_sptr self) -> long"""
        return _blocks_swig2.complex_to_magphase_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_magphase_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_magphase_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_magphase_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_magphase_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_magphase_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_magphase_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_magphase_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_magphase_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_magphase_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_magphase_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_magphase_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_magphase_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_magphase_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_magphase_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_magphase_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_magphase_sptr_message_subscribers(self, which_port)

# Register complex_to_magphase_sptr in _blocks_swig2:
_blocks_swig2.complex_to_magphase_sptr_swigregister(complex_to_magphase_sptr)


complex_to_magphase_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_magphase = complex_to_magphase.make;

class complex_to_real_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_real > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_real_sptr self) -> complex_to_real_sptr
        __init__(complex_to_real_sptr self, complex_to_real p) -> complex_to_real_sptr
        """
        _blocks_swig2.complex_to_real_sptr_swiginit(self, _blocks_swig2.new_complex_to_real_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_real *":
        r"""__deref__(complex_to_real_sptr self) -> complex_to_real"""
        return _blocks_swig2.complex_to_real_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_real_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_real::sptr":
        r"""make(complex_to_real_sptr self, size_t vlen=1) -> complex_to_real_sptr"""
        return _blocks_swig2.complex_to_real_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_real_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_real_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_real_sptr self, int which, int delay)
        declare_sample_delay(complex_to_real_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_real_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_real_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_real_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_real_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_real_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_real_sptr self) -> int"""
        return _blocks_swig2.complex_to_real_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_real_sptr self) -> double"""
        return _blocks_swig2.complex_to_real_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_real_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_real_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_real_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_real_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_real_sptr self) -> bool"""
        return _blocks_swig2.complex_to_real_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_real_sptr self) -> bool"""
        return _blocks_swig2.complex_to_real_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_real_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_real_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_real_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_real_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_real_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_real_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_real_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_real_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_real_sptr self) -> int"""
        return _blocks_swig2.complex_to_real_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_real_sptr self, int m)"""
        return _blocks_swig2.complex_to_real_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_real_sptr self)"""
        return _blocks_swig2.complex_to_real_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_real_sptr self) -> bool"""
        return _blocks_swig2.complex_to_real_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_real_sptr self, int m)"""
        return _blocks_swig2.complex_to_real_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_real_sptr self) -> int"""
        return _blocks_swig2.complex_to_real_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_real_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_real_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_real_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_real_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_real_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_real_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_real_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_real_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_real_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_real_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_real_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_real_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_real_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_real_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_real_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_real_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_real_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_real_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_real_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_real_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_real_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_real_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_real_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_real_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_real_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_real_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_real_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_real_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_real_sptr self) -> float"""
        return _blocks_swig2.complex_to_real_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_real_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_real_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_real_sptr self)"""
        return _blocks_swig2.complex_to_real_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_real_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_real_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_real_sptr self) -> int"""
        return _blocks_swig2.complex_to_real_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_real_sptr self) -> int"""
        return _blocks_swig2.complex_to_real_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_real_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_real_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_real_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_real_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_real_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_real_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_real_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_real_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_real_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_real_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_real_sptr self) -> long"""
        return _blocks_swig2.complex_to_real_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_real_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_real_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_real_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_real_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_real_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_real_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_real_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_real_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_real_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_real_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_real_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_real_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_real_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_real_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_real_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_real_sptr_message_subscribers(self, which_port)

# Register complex_to_real_sptr in _blocks_swig2:
_blocks_swig2.complex_to_real_sptr_swigregister(complex_to_real_sptr)


complex_to_real_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_real = complex_to_real.make;

class complex_to_imag_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_imag > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_imag_sptr self) -> complex_to_imag_sptr
        __init__(complex_to_imag_sptr self, complex_to_imag p) -> complex_to_imag_sptr
        """
        _blocks_swig2.complex_to_imag_sptr_swiginit(self, _blocks_swig2.new_complex_to_imag_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_imag *":
        r"""__deref__(complex_to_imag_sptr self) -> complex_to_imag"""
        return _blocks_swig2.complex_to_imag_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_imag_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_imag::sptr":
        r"""make(complex_to_imag_sptr self, size_t vlen=1) -> complex_to_imag_sptr"""
        return _blocks_swig2.complex_to_imag_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_imag_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_imag_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_imag_sptr self, int which, int delay)
        declare_sample_delay(complex_to_imag_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_imag_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_imag_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_imag_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_imag_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_imag_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_imag_sptr self) -> int"""
        return _blocks_swig2.complex_to_imag_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_imag_sptr self) -> double"""
        return _blocks_swig2.complex_to_imag_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_imag_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_imag_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_imag_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_imag_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_imag_sptr self) -> bool"""
        return _blocks_swig2.complex_to_imag_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_imag_sptr self) -> bool"""
        return _blocks_swig2.complex_to_imag_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_imag_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_imag_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_imag_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_imag_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_imag_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_imag_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_imag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_imag_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_imag_sptr self) -> int"""
        return _blocks_swig2.complex_to_imag_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_imag_sptr self, int m)"""
        return _blocks_swig2.complex_to_imag_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_imag_sptr self)"""
        return _blocks_swig2.complex_to_imag_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_imag_sptr self) -> bool"""
        return _blocks_swig2.complex_to_imag_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_imag_sptr self, int m)"""
        return _blocks_swig2.complex_to_imag_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_imag_sptr self) -> int"""
        return _blocks_swig2.complex_to_imag_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_imag_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_imag_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_imag_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_imag_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_imag_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_imag_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_imag_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_imag_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_imag_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_imag_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_imag_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_imag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_imag_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_imag_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_imag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_imag_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_imag_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_imag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_imag_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_imag_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_imag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_imag_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_imag_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_imag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_imag_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_imag_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_imag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_imag_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_imag_sptr self) -> float"""
        return _blocks_swig2.complex_to_imag_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_imag_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_imag_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_imag_sptr self)"""
        return _blocks_swig2.complex_to_imag_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_imag_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_imag_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_imag_sptr self) -> int"""
        return _blocks_swig2.complex_to_imag_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_imag_sptr self) -> int"""
        return _blocks_swig2.complex_to_imag_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_imag_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_imag_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_imag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_imag_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_imag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_imag_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_imag_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_imag_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_imag_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_imag_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_imag_sptr self) -> long"""
        return _blocks_swig2.complex_to_imag_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_imag_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_imag_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_imag_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_imag_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_imag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_imag_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_imag_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_imag_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_imag_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_imag_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_imag_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_imag_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_imag_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_imag_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_imag_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_imag_sptr_message_subscribers(self, which_port)

# Register complex_to_imag_sptr in _blocks_swig2:
_blocks_swig2.complex_to_imag_sptr_swigregister(complex_to_imag_sptr)


complex_to_imag_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_imag = complex_to_imag.make;

class complex_to_mag_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_mag > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_mag_sptr self) -> complex_to_mag_sptr
        __init__(complex_to_mag_sptr self, complex_to_mag p) -> complex_to_mag_sptr
        """
        _blocks_swig2.complex_to_mag_sptr_swiginit(self, _blocks_swig2.new_complex_to_mag_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_mag *":
        r"""__deref__(complex_to_mag_sptr self) -> complex_to_mag"""
        return _blocks_swig2.complex_to_mag_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_mag_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_mag::sptr":
        r"""make(complex_to_mag_sptr self, size_t vlen=1) -> complex_to_mag_sptr"""
        return _blocks_swig2.complex_to_mag_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_mag_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_mag_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_mag_sptr self, int which, int delay)
        declare_sample_delay(complex_to_mag_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_mag_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_mag_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_mag_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_mag_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_mag_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_mag_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_mag_sptr self) -> double"""
        return _blocks_swig2.complex_to_mag_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_mag_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_mag_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_mag_sptr self) -> bool"""
        return _blocks_swig2.complex_to_mag_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_mag_sptr self) -> bool"""
        return _blocks_swig2.complex_to_mag_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_mag_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_mag_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_mag_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_mag_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_mag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_mag_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_mag_sptr self, int m)"""
        return _blocks_swig2.complex_to_mag_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_mag_sptr self)"""
        return _blocks_swig2.complex_to_mag_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_mag_sptr self) -> bool"""
        return _blocks_swig2.complex_to_mag_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_mag_sptr self, int m)"""
        return _blocks_swig2.complex_to_mag_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_mag_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_mag_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_mag_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_mag_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_mag_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_mag_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_mag_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_mag_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_mag_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_mag_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_mag_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_mag_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_mag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_mag_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_mag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_mag_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_mag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_mag_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_mag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_mag_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_mag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_mag_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_mag_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_mag_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_mag_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_mag_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_mag_sptr self)"""
        return _blocks_swig2.complex_to_mag_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_mag_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_mag_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_mag_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_mag_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_mag_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_mag_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_mag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_mag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_mag_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_mag_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_mag_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_mag_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_mag_sptr self) -> long"""
        return _blocks_swig2.complex_to_mag_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_mag_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_mag_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_mag_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_mag_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_mag_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_mag_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_mag_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_mag_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_mag_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_mag_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_mag_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_mag_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_mag_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_mag_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_mag_sptr_message_subscribers(self, which_port)

# Register complex_to_mag_sptr in _blocks_swig2:
_blocks_swig2.complex_to_mag_sptr_swigregister(complex_to_mag_sptr)


complex_to_mag_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_mag = complex_to_mag.make;

class complex_to_mag_squared_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_mag_squared > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_mag_squared_sptr self) -> complex_to_mag_squared_sptr
        __init__(complex_to_mag_squared_sptr self, complex_to_mag_squared p) -> complex_to_mag_squared_sptr
        """
        _blocks_swig2.complex_to_mag_squared_sptr_swiginit(self, _blocks_swig2.new_complex_to_mag_squared_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_mag_squared *":
        r"""__deref__(complex_to_mag_squared_sptr self) -> complex_to_mag_squared"""
        return _blocks_swig2.complex_to_mag_squared_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_mag_squared_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_mag_squared::sptr":
        r"""make(complex_to_mag_squared_sptr self, size_t vlen=1) -> complex_to_mag_squared_sptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_mag_squared_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_mag_squared_sptr self, int which, int delay)
        declare_sample_delay(complex_to_mag_squared_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_mag_squared_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_mag_squared_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_mag_squared_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_mag_squared_sptr self) -> double"""
        return _blocks_swig2.complex_to_mag_squared_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_mag_squared_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_squared_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_mag_squared_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_squared_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_mag_squared_sptr self) -> bool"""
        return _blocks_swig2.complex_to_mag_squared_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_mag_squared_sptr self) -> bool"""
        return _blocks_swig2.complex_to_mag_squared_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_mag_squared_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_squared_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_mag_squared_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_mag_squared_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_mag_squared_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_mag_squared_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_squared_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_mag_squared_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_mag_squared_sptr self, int m)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_mag_squared_sptr self)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_mag_squared_sptr self) -> bool"""
        return _blocks_swig2.complex_to_mag_squared_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_mag_squared_sptr self, int m)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_mag_squared_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_mag_squared_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_mag_squared_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_mag_squared_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_mag_squared_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_mag_squared_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_mag_squared_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_mag_squared_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_mag_squared_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_mag_squared_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_mag_squared_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_mag_squared_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_mag_squared_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_mag_squared_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_mag_squared_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_mag_squared_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_mag_squared_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_mag_squared_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_mag_squared_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_mag_squared_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_mag_squared_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_mag_squared_sptr self) -> float"""
        return _blocks_swig2.complex_to_mag_squared_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_mag_squared_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_mag_squared_sptr self)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_mag_squared_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_mag_squared_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_mag_squared_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_mag_squared_sptr self) -> int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_mag_squared_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_mag_squared_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_squared_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_mag_squared_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_squared_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_mag_squared_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_mag_squared_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_mag_squared_sptr self) -> long"""
        return _blocks_swig2.complex_to_mag_squared_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_mag_squared_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_mag_squared_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_mag_squared_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_mag_squared_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_mag_squared_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_mag_squared_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_mag_squared_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_mag_squared_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_mag_squared_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_mag_squared_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_mag_squared_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_mag_squared_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_mag_squared_sptr_message_subscribers(self, which_port)

# Register complex_to_mag_squared_sptr in _blocks_swig2:
_blocks_swig2.complex_to_mag_squared_sptr_swigregister(complex_to_mag_squared_sptr)


complex_to_mag_squared_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_mag_squared = complex_to_mag_squared.make;

class complex_to_arg_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::complex_to_arg > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(complex_to_arg_sptr self) -> complex_to_arg_sptr
        __init__(complex_to_arg_sptr self, complex_to_arg p) -> complex_to_arg_sptr
        """
        _blocks_swig2.complex_to_arg_sptr_swiginit(self, _blocks_swig2.new_complex_to_arg_sptr(*args))

    def __deref__(self) -> "gr::blocks::complex_to_arg *":
        r"""__deref__(complex_to_arg_sptr self) -> complex_to_arg"""
        return _blocks_swig2.complex_to_arg_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_complex_to_arg_sptr

    def make(self, vlen: "size_t"=1) -> "gr::blocks::complex_to_arg::sptr":
        r"""make(complex_to_arg_sptr self, size_t vlen=1) -> complex_to_arg_sptr"""
        return _blocks_swig2.complex_to_arg_sptr_make(self, vlen)

    def history(self) -> "unsigned int":
        r"""history(complex_to_arg_sptr self) -> unsigned int"""
        return _blocks_swig2.complex_to_arg_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(complex_to_arg_sptr self, int which, int delay)
        declare_sample_delay(complex_to_arg_sptr self, unsigned int delay)
        """
        return _blocks_swig2.complex_to_arg_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(complex_to_arg_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.complex_to_arg_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(complex_to_arg_sptr self, int multiple)"""
        return _blocks_swig2.complex_to_arg_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(complex_to_arg_sptr self) -> int"""
        return _blocks_swig2.complex_to_arg_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(complex_to_arg_sptr self) -> double"""
        return _blocks_swig2.complex_to_arg_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(complex_to_arg_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_arg_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(complex_to_arg_sptr self) -> uint64_t"""
        return _blocks_swig2.complex_to_arg_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(complex_to_arg_sptr self) -> bool"""
        return _blocks_swig2.complex_to_arg_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(complex_to_arg_sptr self) -> bool"""
        return _blocks_swig2.complex_to_arg_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(complex_to_arg_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.complex_to_arg_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(complex_to_arg_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.complex_to_arg_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(complex_to_arg_sptr self, std::string level)"""
        return _blocks_swig2.complex_to_arg_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(complex_to_arg_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_arg_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(complex_to_arg_sptr self) -> int"""
        return _blocks_swig2.complex_to_arg_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(complex_to_arg_sptr self, int m)"""
        return _blocks_swig2.complex_to_arg_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(complex_to_arg_sptr self)"""
        return _blocks_swig2.complex_to_arg_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(complex_to_arg_sptr self) -> bool"""
        return _blocks_swig2.complex_to_arg_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(complex_to_arg_sptr self, int m)"""
        return _blocks_swig2.complex_to_arg_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(complex_to_arg_sptr self) -> int"""
        return _blocks_swig2.complex_to_arg_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(complex_to_arg_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_arg_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(complex_to_arg_sptr self, long max_output_buffer)
        set_max_output_buffer(complex_to_arg_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.complex_to_arg_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(complex_to_arg_sptr self, int i) -> long"""
        return _blocks_swig2.complex_to_arg_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(complex_to_arg_sptr self, long min_output_buffer)
        set_min_output_buffer(complex_to_arg_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.complex_to_arg_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(complex_to_arg_sptr self, int which) -> float
        pc_input_buffers_full(complex_to_arg_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_arg_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(complex_to_arg_sptr self, int which) -> float
        pc_input_buffers_full_avg(complex_to_arg_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_arg_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(complex_to_arg_sptr self, int which) -> float
        pc_input_buffers_full_var(complex_to_arg_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_arg_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(complex_to_arg_sptr self, int which) -> float
        pc_output_buffers_full(complex_to_arg_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_arg_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(complex_to_arg_sptr self, int which) -> float
        pc_output_buffers_full_avg(complex_to_arg_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_arg_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(complex_to_arg_sptr self, int which) -> float
        pc_output_buffers_full_var(complex_to_arg_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.complex_to_arg_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(complex_to_arg_sptr self) -> float"""
        return _blocks_swig2.complex_to_arg_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(complex_to_arg_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.complex_to_arg_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(complex_to_arg_sptr self)"""
        return _blocks_swig2.complex_to_arg_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(complex_to_arg_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.complex_to_arg_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(complex_to_arg_sptr self) -> int"""
        return _blocks_swig2.complex_to_arg_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(complex_to_arg_sptr self) -> int"""
        return _blocks_swig2.complex_to_arg_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(complex_to_arg_sptr self, int priority) -> int"""
        return _blocks_swig2.complex_to_arg_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(complex_to_arg_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_arg_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(complex_to_arg_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_arg_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(complex_to_arg_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_arg_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(complex_to_arg_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.complex_to_arg_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(complex_to_arg_sptr self) -> long"""
        return _blocks_swig2.complex_to_arg_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(complex_to_arg_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.complex_to_arg_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(complex_to_arg_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.complex_to_arg_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(complex_to_arg_sptr self) -> std::string"""
        return _blocks_swig2.complex_to_arg_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(complex_to_arg_sptr self, std::string name)"""
        return _blocks_swig2.complex_to_arg_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(complex_to_arg_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.complex_to_arg_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(complex_to_arg_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_arg_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(complex_to_arg_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_arg_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(complex_to_arg_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.complex_to_arg_sptr_message_subscribers(self, which_port)

# Register complex_to_arg_sptr in _blocks_swig2:
_blocks_swig2.complex_to_arg_sptr_swigregister(complex_to_arg_sptr)


complex_to_arg_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
complex_to_arg = complex_to_arg.make;

class conjugate_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::blocks::conjugate_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(conjugate_cc_sptr self) -> conjugate_cc_sptr
        __init__(conjugate_cc_sptr self, conjugate_cc p) -> conjugate_cc_sptr
        """
        _blocks_swig2.conjugate_cc_sptr_swiginit(self, _blocks_swig2.new_conjugate_cc_sptr(*args))

    def __deref__(self) -> "gr::blocks::conjugate_cc *":
        r"""__deref__(conjugate_cc_sptr self) -> conjugate_cc"""
        return _blocks_swig2.conjugate_cc_sptr___deref__(self)
    __swig_destroy__ = _blocks_swig2.delete_conjugate_cc_sptr

    def make(self) -> "gr::blocks::conjugate_cc::sptr":
        r"""make(conjugate_cc_sptr self) -> conjugate_cc_sptr"""
        return _blocks_swig2.conjugate_cc_sptr_make(self)

    def history(self) -> "unsigned int":
        r"""history(conjugate_cc_sptr self) -> unsigned int"""
        return _blocks_swig2.conjugate_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(conjugate_cc_sptr self, int which, int delay)
        declare_sample_delay(conjugate_cc_sptr self, unsigned int delay)
        """
        return _blocks_swig2.conjugate_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(conjugate_cc_sptr self, int which) -> unsigned int"""
        return _blocks_swig2.conjugate_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(conjugate_cc_sptr self, int multiple)"""
        return _blocks_swig2.conjugate_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(conjugate_cc_sptr self) -> int"""
        return _blocks_swig2.conjugate_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(conjugate_cc_sptr self) -> double"""
        return _blocks_swig2.conjugate_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(conjugate_cc_sptr self) -> uint64_t"""
        return _blocks_swig2.conjugate_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(conjugate_cc_sptr self) -> uint64_t"""
        return _blocks_swig2.conjugate_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(conjugate_cc_sptr self) -> bool"""
        return _blocks_swig2.conjugate_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(conjugate_cc_sptr self) -> bool"""
        return _blocks_swig2.conjugate_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(conjugate_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _blocks_swig2.conjugate_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(conjugate_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _blocks_swig2.conjugate_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(conjugate_cc_sptr self, std::string level)"""
        return _blocks_swig2.conjugate_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(conjugate_cc_sptr self) -> std::string"""
        return _blocks_swig2.conjugate_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(conjugate_cc_sptr self) -> int"""
        return _blocks_swig2.conjugate_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(conjugate_cc_sptr self, int m)"""
        return _blocks_swig2.conjugate_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(conjugate_cc_sptr self)"""
        return _blocks_swig2.conjugate_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(conjugate_cc_sptr self) -> bool"""
        return _blocks_swig2.conjugate_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(conjugate_cc_sptr self, int m)"""
        return _blocks_swig2.conjugate_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(conjugate_cc_sptr self) -> int"""
        return _blocks_swig2.conjugate_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(conjugate_cc_sptr self, int i) -> long"""
        return _blocks_swig2.conjugate_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(conjugate_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(conjugate_cc_sptr self, int port, long max_output_buffer)
        """
        return _blocks_swig2.conjugate_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(conjugate_cc_sptr self, int i) -> long"""
        return _blocks_swig2.conjugate_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(conjugate_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(conjugate_cc_sptr self, int port, long min_output_buffer)
        """
        return _blocks_swig2.conjugate_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(conjugate_cc_sptr self, int which) -> float
        pc_input_buffers_full(conjugate_cc_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.conjugate_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(conjugate_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(conjugate_cc_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.conjugate_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(conjugate_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(conjugate_cc_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.conjugate_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(conjugate_cc_sptr self, int which) -> float
        pc_output_buffers_full(conjugate_cc_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.conjugate_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(conjugate_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(conjugate_cc_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.conjugate_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(conjugate_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(conjugate_cc_sptr self) -> pmt_vector_float
        """
        return _blocks_swig2.conjugate_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(conjugate_cc_sptr self) -> float"""
        return _blocks_swig2.conjugate_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(conjugate_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _blocks_swig2.conjugate_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(conjugate_cc_sptr self)"""
        return _blocks_swig2.conjugate_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(conjugate_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _blocks_swig2.conjugate_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(conjugate_cc_sptr self) -> int"""
        return _blocks_swig2.conjugate_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(conjugate_cc_sptr self) -> int"""
        return _blocks_swig2.conjugate_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(conjugate_cc_sptr self, int priority) -> int"""
        return _blocks_swig2.conjugate_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(conjugate_cc_sptr self) -> std::string"""
        return _blocks_swig2.conjugate_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(conjugate_cc_sptr self) -> std::string"""
        return _blocks_swig2.conjugate_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(conjugate_cc_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.conjugate_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(conjugate_cc_sptr self) -> io_signature_sptr"""
        return _blocks_swig2.conjugate_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(conjugate_cc_sptr self) -> long"""
        return _blocks_swig2.conjugate_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(conjugate_cc_sptr self) -> basic_block_sptr"""
        return _blocks_swig2.conjugate_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(conjugate_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _blocks_swig2.conjugate_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(conjugate_cc_sptr self) -> std::string"""
        return _blocks_swig2.conjugate_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(conjugate_cc_sptr self, std::string name)"""
        return _blocks_swig2.conjugate_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(conjugate_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _blocks_swig2.conjugate_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(conjugate_cc_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.conjugate_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(conjugate_cc_sptr self) -> swig_pmt_ptr"""
        return _blocks_swig2.conjugate_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(conjugate_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _blocks_swig2.conjugate_cc_sptr_message_subscribers(self, which_port)

# Register conjugate_cc_sptr in _blocks_swig2:
_blocks_swig2.conjugate_cc_sptr_swigregister(conjugate_cc_sptr)


conjugate_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
conjugate_cc = conjugate_cc.make;



